// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef CVS_ACTION_H
#define CVS_ACTION_H

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include <string>
#include <windows.h>
#include <stdio.h>
#include "MakeArgs.h"
#include "../TortoiseAct/TortoiseDialogs.h"
#include "../Utils/Thread.h"

class ProgressDialogAbstract;
class ThreadSafeProgress;

class CVSActionPipe
{
public:
	enum
	{
		INPUT_IS_FILE = 0x0001,
		OUTPUT_IS_FILE = 0x0002
	};
	CVSActionPipe(int flags = 0);
	virtual ~CVSActionPipe();

	virtual HANDLE InputHandle() const { return inputHandle; }
	virtual HANDLE OutputHandle() const { return outputHandle; }
	virtual FILE * InputFile() const { return inputFile; }
	virtual FILE * OutputFile() const { return outputFile; }
	virtual void CloseInput();

    virtual void TextIn(std::string textToAdd);
    virtual std::string TextOut();

protected:
	int flags;
	HANDLE inputHandle;
	HANDLE outputHandle;
	FILE *inputFile;
	FILE *outputFile;
    std::string text;
    HANDLE mutex;
};

class CVSAction
{
public:
	// The progress bar appears for the duration of an instantiation of CVSAction
	enum
	{
		PIPE_STDOUT = 0x0001,
		HIDE_PROGRESS = 0x0002
	};
	CVSAction(int flags = 0);
	~CVSAction();
	bool Command(std::string currentDirectory, MakeArgs& args);
	void SetCVSRoot(const std::string& protocol, const std::string& userName, const std::string& serverMachine, const std::string& repositoryDirectory);
	void SetPassword(const std::string& password); // don't need to call this - the user will be prompted if you don't
	void SetStreamFile(const std::string& file);
	void SetCloseIfOK(bool closeIfOK);
	void SetCloseAnyway(bool closeIfOK);
	
	static std::string CalcCVSRoot(const std::string& protocol, const std::string& userName, const std::string& serverMachine, const std::string& repositoryDirectory);

	FILE * ConsoleOutput() const  { return consoleout_pipe->OutputFile(); }
	void CloseConsoleOutput();

	bool AtLeastOneSuccess();

	ProgressDialogAbstract* GetProgressDialog();

private:
	int flags;

	static CVSActionPipe * consoleout_pipe;

	bool DoCommand(std::string currentDirectory, MakeArgs args);
	bool DoCommandThread(std::string currentDirectory, MakeArgs args);
	std::string GetLibName(const std::string& currentDirectory);
	void ThreadMain();

    static DWORD WINAPI PipeThread(LPVOID pActionPipe);
    static void SetEnvironment(std::string name, std::string value);
    static bool PipeToGUI(CVSActionPipe *pPipe, std::string &store);

	static bool ourInUse;
	static std::string ourProtocol;
	static std::string ourUserName;
	static std::string ourServerMachine;
	static std::string ourPassword;
	static std::string ourRepositoryDirectory;
	static std::string ourCVSROOT;

	static ThreadSafeProgress* ourThreadSafeProgressDialog;
	ProgressDialogAbstract* myProgressDialog;

	static DWORD ourLastTick;
	static std::string ourStreamingToFile;
	static std::string ourErrorStore;
	static std::string ourOutputStore;
	bool myCloseIfOK;
	bool myCloseAnyway;

	int myUses;
	int myErrors;

	Thread<CVSAction> myThread;
	bool myThreadResult;
	std::string myThreadDirectory;
	MakeArgs myThreadArgs;
};


#endif // CVS_ACTION_H
