// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef CVS_STATUS_H
#define CVS_STATUS_H

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include <string>
#include <vector>
#include <map>
#include <time.h>
#ifdef __BORLANDC__
	//Types needed for time functions
	using std::time_t;
#endif
#include "CvsEntries.h"

class ProgressDialog;

class CVSStatus
{
public:
	enum FileStatus
	{
		STATUS_NOWHERENEAR_CVS,
		STATUS_INCVS,
		STATUS_CHANGED,
		STATUS_CONFLICT,
		STATUS_NOTINCVS,
		STATUS_OUTERDIRECTORY, // Special for top level directories that contain stuff which is in CVS
	};

	static FileStatus GetFileStatus(const std::string& filename);
	static FileStatus GetFileStatusRecursive(const std::string& filename);
	static void RecursiveScan(std::string directory, std::vector<std::string>& modified, std::vector<std::string>& added, std::vector<std::string>& removed);

	static bool IsFileBinary(const std::string& filename);
	static bool IsFileText(const std::string& filename);
	static bool InCVSHasSubdirs(const std::string& filename);

	static std::string CVSRootForPath(std::string path);
	static std::string RootServer(const std::string& cvsroot);
	static std::string RootProtocol(const std::string& cvsroot);
	static std::string RootUserName(const std::string& cvsroot);
	static std::string ServerPath(std::string path);

	static bool HasRevisionNumber(const std::string& filename);
	static bool HasTimestamp(const std::string& filename);
	static bool HasStickyTag(const std::string& filename);
	static bool HasStickyDate(const std::string& filename);
	static bool HasOptions(const std::string& filename);

	static std::string GetRevisionNumber(const std::string& filename);
	static std::string GetTimestamp(const std::string& filename);
	static std::string GetStickyTag(const std::string& filename);
	static std::string GetStickyDate(const std::string& filename);
	static std::string GetOptions(const std::string& filename);

private:
	static EntnodeData* GetEntries(const std::string& filename);
	static EntnodeData* GetEntriesUncached(const std::string& filename);

	static std::map<std::string, ENTNODE> ourLastEntries;
	static std::map<std::string, EntnodeData*> ourEntnodeCache;
	static time_t ourEntnodeCacheTimeStamp;
	static std::string ourEntnodeCacheDirectory;
	static bool ourEntnodeErased;

	static time_t ourRecursiveStatusCacheTimeStamp;
	static FileStatus ourRecursiveStatus;
	static std::string ourRecursiveStatusFilename;
	static FileStatus GetFileStatusRecursiveUncached(const std::string& filename);

	static bool RecursiveHasChangedFiles(std::string scanDirectory);
};
	

#endif // CVS_STATUS_H
