// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include "MakeArgs.h"
#include "../Utils/TortoiseRegistry.h"

MakeArgs::MakeArgs()
{
	myArgs.push_back("cvs");

	int quietness = TortoiseRegistry::ReadInteger("Quietness", 1);
	if (quietness == 1)
		myArgs.push_back("-q");
	else if (quietness == 2)
		myArgs.push_back("-Q");
	// otherwise, noisy

	int compression = TortoiseRegistry::ReadInteger("Compression Level", 2);
	if (compression == 1)
		myArgs.push_back("-z3");
	else if (compression == 2)
		myArgs.push_back("-z6");
	else if (compression == 3)
		myArgs.push_back("-z9");
	// otherwise, no compression

	myMainArg = myArgs.size();
}

void MakeArgs::add(const std::string& arg)
{
	myArgs.push_back(arg);
}

int MakeArgs::argc()
{
	return myArgs.size();
}

char** MakeArgs::argv()
{
	myArgPointers.clear();
	int n = myArgs.size();
	for (int i = 0; i < n; ++i)
	{
		myArgPointers.push_back(myArgs[i].c_str());
	}
	myArgPointers.push_back(NULL);
	return const_cast<char **>(&myArgPointers[0]);
}

// Remake the arguments, into the form they would need to 
// be typed into a shell.  Quotes are put round arguments
// which contain spaces.
std::string MakeArgs::requoted()
{
	std::string remade;
	int n = myArgs.size();
	for (int i = 0; i < n; ++i)
	{
		bool quote = false;
		if (myArgs[i].find_first_of(' ') != std::string::npos)
			quote = true;
		if (myArgs[i].empty())
			quote = true;

		if (quote)
			remade += "\"";
		remade += myArgs[i];
		if (quote)
			remade += "\"";
		if (i != n - 1)
			remade += " ";
	}

	return remade;
}

std::string MakeArgs::main_command()
{
	return myArgs[myMainArg];
}