// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "HasMenu.h"
#include "../CVSGlue/CVSStatus.h"
#include "../Utils/PathUtils.h"

bool HasMenu(int flags, const std::vector<std::string>& files)
{
	// Override everything
	if (flags & ALWAYS_HAS)
		return true;

	// Check there is exactly one
	if (flags & EXACTLY_ONE)
		if (files.size() != 1)
			return false;

	// Check the TYPE criteria
	bool allMustMatch = (flags & ALL_MUST_MATCH_TYPE) ? true : false;
	bool match = allMustMatch;
	for (unsigned int i = 0; i < files.size(); ++i)
	{
		// Clever bit to see if ALL must match type,
		// or just at least one:

		// ALL: Stop if one breaks the rule
		if (allMustMatch && !match)
			break;

		// AT LEAST ONE: Stop if one meets the rule
		if (!allMustMatch && match)
			break;

		// Determine whether this file meets the criteria
		match = false;

		// Do not touch root paths, e.g. A:\; this
		// prevents drive access to drives that may
		// not contain any media (on Win2K).
		if (IsRootPath(files[i]))
				continue;

		// Nothing matches if it is called "CVS"
		// (e.g. the CVS directory)
		std::string last = ExtractLastPart(files[i]);
		if (last == "CVS" || last == "Cvs")
			continue; // false

		// Check directory/file type
		bool directory = IsDirectory(files[i].c_str());
		if (flags & TYPE_DIRECTORY)
			if (!directory)
				continue; // false
		if (flags & TYPE_FILE)
			if (directory)
				continue; // false

		// Special case for non-recursive CVS Remove
		// TODO: This doesn't make sense to me, as you can't remove
		// directories in CVS anyway  What was I thinking when I wrote
		// this?
		if (flags & TYPE_REMOVEABLE)
			if (CVSStatus::InCVSHasSubdirs(files[i]))
				continue; // false
		
		// Check the ones that correspond to CVSStatus::STATUS_
		CVSStatus::FileStatus status = CVSStatus::GetFileStatus(files[i]);
		match =
			(CVSStatus::STATUS_NOTINCVS == status && (TYPE_NOTINCVS & flags)) ||
			(CVSStatus::STATUS_NOWHERENEAR_CVS == status && (TYPE_NOWHERENEAR_CVS & flags)) ||
			(CVSStatus::STATUS_INCVS == status && (TYPE_INCVS & flags)) ||
			(CVSStatus::STATUS_CHANGED == status && (TYPE_CHANGED & flags)) ||
			(CVSStatus::STATUS_CONFLICT == status && (TYPE_CONFLICT & flags)) ||
			(CVSStatus::STATUS_OUTERDIRECTORY == status && (TYPE_OUTERDIRECTORY & flags));
	}

	return match;
}

