// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef HAS_MENU_H
#define HAS_MENU_H

#include <vector>
#include <string>

enum MenuFlags
{
	// General parameters:
	ON_SUB_MENU = 4,

	// Whether the menu is present:

	EXACTLY_ONE = 1, // There must be only one file/directory
	ALWAYS_HAS = 2, // The menu is always present

	// By default, at least one file must match the TYPE_
	// flags below.  Use this to force it to be all files.
	ALL_MUST_MATCH_TYPE = 131072,

	// These correspond to CVSStatus::STATUS_*
	// If several are present, it counts as an "or" between them
	TYPE_NOTINCVS = 256,
	TYPE_NOWHERENEAR_CVS = 512,
	TYPE_INCVS = 1024,
	TYPE_CHANGED = 2048,
	TYPE_CONFLICT = 4096,
	TYPE_OUTERDIRECTORY = 8192, // Special for top level directories that contain stuff which is in CVS
	
	// If multiple ones of these are present, then they
	// must all be true (along with the STATUS_ type flags above)
	TYPE_DIRECTORY = 16384,
	TYPE_FILE = 32768,
	TYPE_REMOVEABLE = 65536, // Special, checks nothing inside so can be removed (without recursion!)

};

// Tests the given files against the flags to see if a particular
// menu item should be present.
bool HasMenu(int flags, const std::vector<std::string>& files);

#endif
