// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Ben Campbell
// <ben.campbell@creaturelabs.com> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "BranchTagDlg.h"
#include "WxwHelpers.h"

//static
bool DoTagDialog( std::string& tagname, bool& checkunmodified )
{
	BranchTagDlg* dlg = new BranchTagDlg(BranchTagDlg::modeTag);
	bool ret = (dlg->ShowModal() == wxID_OK);

	if (ret)
	{
		checkunmodified = dlg->myCheckUnmodifiedButton->GetValue();
		tagname = dlg->myTagEntry->GetValue();
	}

	DestroyDialog(dlg);
	return ret;
}

//static
bool DoBranchDialog( std::string& branchname, bool& checkunmodified )
{
	BranchTagDlg* dlg = new BranchTagDlg(BranchTagDlg::modeBranch);
	bool ret = (dlg->ShowModal() == wxID_OK);

	if (ret)
	{
		checkunmodified = dlg->myCheckUnmodifiedButton->GetValue();
		branchname = dlg->myTagEntry->GetValue();
	}

	DestroyDialog(dlg);
	return ret;
}



BranchTagDlg::BranchTagDlg( int mode )
	: wxDialog(NULL, -1, wxString(mode == modeTag ? "TortoiseCVS - Create Tag" : "TortoiseCVS - Create Branch"),
	wxDefaultPosition, wxSize(400,300), wxDEFAULT_DIALOG_STYLE)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	// Top part with tag entry field
	wxBoxSizer *sizerTagEntry= new wxBoxSizer(wxHORIZONTAL);
	wxStaticText *label = new wxStaticText(this, -1, mode == modeTag ? "Tag:" : "Branch:");
    myTagEntry = new wxTextCtrl(this, -1, "");
	myTagEntry->SetToolTip( "Name of tag or branch to create." );

    sizerTagEntry->Add(label, 0, wxALIGN_CENTER_VERTICAL, 5);
    sizerTagEntry->Add(myTagEntry, 1, wxGROW | wxALL, 5);
	myTagEntry->SetFocus();

	// Check box
	myCheckUnmodifiedButton = new wxCheckBox(this, -1,
		"Check that the files are unmodified");
	myCheckUnmodifiedButton->SetValue( true );
	myCheckUnmodifiedButton->SetToolTip( "Ensures there are no uncommited "
		"changes before performing the operation" );

	// information text
	std::string inf;
	if( mode == modeTag )
	{
		inf = "The tag will be assigned to the repository versions "
			"nearest to your working sources.\n"
			"The operation is performed directly upon the repository.\n";
	}
	else
	{
		inf = "The branch will be applied to the repository versions "
			"nearest to your working sources.\n"
			"The operation is performed directly upon the repository.\n"
			"Your working sources remain on the original branch.\n";
	}

	inf += "\nA tag must start with a letter, and may contain letters, "
	"digits, \"-\" (dash) and \"_\" (underscore) only.\n"
	"In particular, this means no dots, and no spaces.";


	wxStaticText *infoText = new wxStaticText( this, -1, inf.c_str(),
		wxDefaultPosition, wxDefaultSize );

	// OK/Cancel button
	wxBoxSizer *sizerConfirm = new wxBoxSizer(wxHORIZONTAL);
    wxButton* ok = new wxButton(this, wxID_OK, "OK");
	ok->SetDefault();
    wxButton* cancel = new wxButton(this, wxID_CANCEL, "Cancel");
	sizerConfirm->Add(ok, 0, wxGROW | wxALL, 5);
    sizerConfirm->Add(cancel, 0, wxGROW | wxALL, 5);

	// Main box with everything in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	sizerTop->Add(sizerTagEntry, 0, wxGROW | wxALL, 8);
	sizerTop->Add(myCheckUnmodifiedButton, 0, wxLEFT | wxRIGHT, 8);
	sizerTop->Add(infoText, 0, wxGROW | wxALL, 8);
	sizerTop->Add(sizerConfirm, 0, wxALIGN_CENTER | wxALL, 0);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
    sizerTop->Fit(this);
}
