// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "LogConfigDialog.h"
#include "WxwHelpers.h"

bool DoLogConfigDialog( std::string& urlStubPrefix, std::string& urlStubSuffix, const std::string& message, const std::string& regKey)
{
	LogConfigDialog* dlg = new LogConfigDialog(const_cast<char*>(urlStubPrefix.c_str()), const_cast<char*>(urlStubSuffix.c_str()), 
		const_cast<char*>(message.c_str()), regKey);

	bool ret = dlg->ShowModal() == wxID_OK;

	if (ret)
	{
		WriteComboList(dlg->myPrefixCombo, "LogConfig " + dlg->myRegKey + " URLPrefix History");
		WriteComboList(dlg->mySuffixCombo, "LogConfig " + dlg->myRegKey + " URLSuffix History");

		urlStubPrefix = dlg->myURLPrefix;
		urlStubSuffix = dlg->myURLSuffix;
	}

	DestroyDialog(dlg);
	return ret;
}

LogConfigDialog::LogConfigDialog(char* initialURLPrefix, char* initialURLSuffix, char* message, const std::string& regKey)
	: wxDialog(NULL, -1, wxString("TortoiseCVS - Web Log Configuration"),
	wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	myRegKey = regKey;
	myOK = false;

	// Description
	// wxTextCtrl *label1 = new wxTextCtrl(this, -1, message, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
	wxStaticText *label1 = new wxStaticText(this, -1, message, wxDefaultPosition, wxSize(300,100));	

	// Web log and prefix
	wxBoxSizer *sizerURLPrefix = new wxBoxSizer(wxHORIZONTAL);
	myRadioURL = new wxRadioButton(this, -1, "Web log URL prefix:", wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
	myRadioURL->SetValue(true);
	myPrefixCombo = MakeComboList(this, "LogConfig " + myRegKey + " URLPrefix History", initialURLPrefix);
    sizerURLPrefix->Add(myRadioURL, 0, wxALIGN_CENTER_VERTICAL | wxGROW | wxALL, 0);
    sizerURLPrefix->Add(myPrefixCombo, 2, wxGROW | wxALL, 5);
	myPrefixCombo->SetFocus();

	// Suffix and sizer
	wxStaticText* label2 = new wxStaticText(this, -1, "Suffix:");
	mySuffixCombo = MakeComboList(this, "LogConfig " + myRegKey + " URLSuffix History", initialURLSuffix);
    sizerURLPrefix->Add(label2, 0, wxALIGN_CENTER_VERTICAL | wxEXPAND | wxALL, 5);
    sizerURLPrefix->Add(mySuffixCombo, 1, wxGROW | wxALL, 5);

	// Scan again
	myRadioScanAgain= new wxRadioButton(this, -1, "Automatically scan the server for the URL (again)");

	// Vanilla
	myRadioVanilla = new wxRadioButton(this, -1, "Plain text log (if your server doesn't support web log)");

	// OK/Cancel button
	wxBoxSizer *sizerConfirm = new wxBoxSizer(wxHORIZONTAL);
    wxButton* ok = new wxButton(this, wxID_OK, "OK");
	ok->SetDefault();
    wxButton* cancel = new wxButton(this, wxID_CANCEL, "Cancel");
    sizerConfirm->Add(ok, 0, wxGROW | wxALL, 5); 
    sizerConfirm->Add(cancel, 0, wxGROW | wxALL, 5);

	// Main vertical with text and button in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	sizerTop->Add(label1, 1, wxALIGN_LEFT | wxGROW | wxALL, 5);
	sizerTop->Add(sizerURLPrefix, 0, wxALIGN_LEFT | wxGROW | wxLEFT | wxRIGHT, 5);
	sizerTop->Add(myRadioVanilla, 0, wxALIGN_LEFT | wxGROW | wxLEFT | wxRIGHT, 5);
	sizerTop->Add(myRadioScanAgain, 0, wxALIGN_LEFT | wxGROW | wxLEFT | wxRIGHT, 5);
	sizerTop->Add(sizerConfirm, 0, wxALIGN_CENTER | wxALL, 0);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
    sizerTop->Fit(this);

	// Wire it all up
	Connect(myRadioURL->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED , reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_RadioChanged));
	Connect(myRadioVanilla->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED , reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_RadioChanged));
	Connect(myRadioScanAgain->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED , reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_RadioChanged));
	Connect(myPrefixCombo->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_TextChanged));
	Connect(myPrefixCombo->GetId(), wxEVT_COMMAND_TEXT_UPDATED, reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_TextChanged));
	Connect(mySuffixCombo->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_TextChanged));
	Connect(mySuffixCombo->GetId(), wxEVT_COMMAND_TEXT_UPDATED, reinterpret_cast<wxObjectEventFunction>(&LogConfigDialog::cb_TextChanged));

	// enable/disable appropriate widgets
	UpdateSensitivity();
	UpdateText();
}

void LogConfigDialog::cb_RadioChanged()
{
	UpdateSensitivity();
	UpdateText();
}

void LogConfigDialog::cb_TextChanged()
{
	UpdateText();
}

// read out url and store in myURL
void LogConfigDialog::UpdateText()
{
	if (myRadioURL->GetValue())
	{
		myURLPrefix = myPrefixCombo->GetValue();
		myURLSuffix = mySuffixCombo->GetValue();
	}
	else
	{
		myURLPrefix = "";
		myURLSuffix = "";
	}

	if (myRadioVanilla->GetValue())
	{
		myURLPrefix = "plainlog";
		myURLSuffix = "";
	}
	if (myRadioScanAgain->GetValue())
	{
		myURLPrefix = "scanagain";
		myURLSuffix = "";
	}
}

void LogConfigDialog::UpdateSensitivity()
{
	bool state = myRadioURL->GetValue();
	myPrefixCombo->Enable(state);
	mySuffixCombo->Enable(state);
}

