// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "MakeModuleDialog.h"
#include "WxwHelpers.h"

bool DoMakeModuleDialog(std::string& protocol, std::string& userName, std::string& serverMachine, std::string& repositoryDirectory, const std::string& module)
{
	MakeModuleDialog* makeModuleDialog = new MakeModuleDialog(module);
	int modal = makeModuleDialog->ShowModal();
	bool ret = (modal == wxID_OK);

	// Allow page a chance to save settings
	if (ret)
		makeModuleDialog->myModuleBasicsPage->OKClicked();

	protocol = makeModuleDialog->myModuleBasicsPage->GetProtocol();
	userName = makeModuleDialog->myModuleBasicsPage->GetUsername();
	serverMachine = makeModuleDialog->myModuleBasicsPage->GetServer();
	repositoryDirectory = makeModuleDialog->myModuleBasicsPage->GetDirectory();

	// Delete the dialog
	DestroyDialog(makeModuleDialog);

	return ret;
}

MakeModuleDialog::MakeModuleDialog(const std::string& module)
	: wxDialog(NULL, -1, wxString("TortoiseCVS - Make New Module"),
	wxDefaultPosition, wxSize(400,300), wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	// Main notebook
	wxNotebook *noteBook = new wxNotebook(this, -1);
	myModuleBasicsPage = new ModuleBasicsPage(noteBook);
	myModuleBasicsPage->SetFixedModule(module);
	noteBook->AddPage(myModuleBasicsPage, "Module");

	// OK/Cancel button
	wxBoxSizer *sizerConfirm = new wxBoxSizer(wxHORIZONTAL);
    wxButton* ok = new wxButton(this, wxID_OK, "OK");
	ok->SetDefault();
    wxButton* cancel = new wxButton(this, wxID_CANCEL, "Cancel");
    sizerConfirm->Add(ok, 0, wxGROW | wxALL, 5);
    sizerConfirm->Add(cancel, 0, wxGROW | wxALL, 5);

	// Main box with text and button in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	sizerTop->Add(noteBook, 1, wxGROW | wxALL, 3);
	sizerTop->Add(sizerConfirm, 0, wxALIGN_RIGHT | wxALL, 0);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
    sizerTop->Fit(this);
}
