// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "ModuleBasicsPage.h"
#include "../CVSGlue/CVSAction.h"
#include "../Utils/TortoiseRegistry.h"
#include <windows.h> // get user name
#include <lmcons.h> // get user name
#include "WxwHelpers.h"

ModuleBasicsPage::ModuleBasicsPage(wxNotebook* notebook)
	: wxPanel(notebook)
{
	myProtocol = "pserver";

	// Two vertical columns
	wxBoxSizer *sizerOuter = new wxBoxSizer(wxHORIZONTAL);
	wxBoxSizer *sizerLeft = new wxBoxSizer(wxVERTICAL);
	wxBoxSizer *sizerRight = new wxBoxSizer(wxVERTICAL);
	sizerOuter->Add(sizerLeft, 0, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	sizerOuter->Add(sizerRight, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
    SetAutoLayout(TRUE);
	SetSizer(sizerOuter);

	// Labels
	wxStaticText *label1 = new wxStaticText(this, -1, "Protocol:");
	wxStaticText *label2 = new wxStaticText(this, -1, "Server:");
	wxStaticText *label3 = new wxStaticText(this, -1, "Repository directory:");
	wxStaticText *label4 = new wxStaticText(this, -1, "User name:");
	wxStaticText *label5 = new wxStaticText(this, -1, "Module:");
	wxStaticText *label6 = new wxStaticText(this, -1, "CVSROOT:");
	sizerLeft->Add(label1, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	sizerLeft->Add(label2, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	sizerLeft->Add(label3, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL, 5);
	sizerLeft->Add(label4, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	sizerLeft->Add(label5, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	sizerLeft->Add(label6, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);

	// Input widgets

	// Protocols
	const int protocolCount = 5;
	wxString protocols[protocolCount] = 
	{
		"Internet (Unix pserver)",
		"Internet (NT server)",
		"Internet (secure shell)",
		"Internet (remote shell)",
		"Locally mounted directory"
	};
	myProtocolMenu = new wxChoice(this, -1, wxDefaultPosition, wxDefaultSize, protocolCount, protocols);
	sizerRight->Add(myProtocolMenu, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	ReadOptionMenu(myProtocolMenu, "Checkout Protocol Default");
	
	myServerCombo = MakeComboList(this, "Checkout Server History");
	sizerRight->Add(myServerCombo, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);

	myDirectoryCombo = MakeComboList(this, "Checkout Directory History");
	sizerRight->Add(myDirectoryCombo, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);

	// Use their login name under Windows as a sensible default
	std::string userDefault;
	char userName[UNLEN + 1];
	DWORD len = UNLEN;
	if (GetUserName(userName, &len))
	  userDefault = userName;
	myUsernameCombo = MakeComboList(this, "Checkout Username History", userDefault);
	sizerRight->Add(myUsernameCombo, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);

	myModuleCombo = MakeComboList(this,  "Checkout Module History");
	sizerRight->Add(myModuleCombo, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	myModuleCombo->SetFocus();

	myCVSROOTLabel = new wxStaticText(this, -1, "");
	sizerRight->Add(myCVSROOTLabel, 1, wxGROW | wxALL | wxALIGN_CENTER_VERTICAL , 5);
	
	// Wiring 
	Connect(myProtocolMenu->GetId(), wxEVT_COMMAND_CHOICE_SELECTED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myServerCombo->GetId(), wxEVT_COMMAND_TEXT_UPDATED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myServerCombo->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myDirectoryCombo->GetId(), wxEVT_COMMAND_TEXT_UPDATED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myDirectoryCombo->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myModuleCombo->GetId(), wxEVT_COMMAND_TEXT_UPDATED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);
	Connect(myModuleCombo->GetId(), wxEVT_COMMAND_COMBOBOX_SELECTED, (wxObjectEventFunction) &ModuleBasicsPage::Refresh);

	// Tooltips
	const char *tip1 = "How your machine should connect to the CVS repository";
	const char *tip2 = "The computer that the CVS repository is on";
	const char *tip3 = "Location of the repository on the server";
	const char *tip4 = "Location of the repository on the server";
	const char *tip5 = "The name of the module that you want to fetch";
	const char *tip6 = "Technical details of the CVSROOT which TortoiseCVS is going to use";

	myProtocolMenu->SetToolTip(tip1);
	myServerCombo->SetToolTip(tip2);
	myDirectoryCombo->SetToolTip(tip3);
	myUsernameCombo->SetToolTip(tip4);
	myModuleCombo->SetToolTip(tip5);
	myCVSROOTLabel->SetToolTip(tip6);

	label1->SetToolTip(tip1);
	label2->SetToolTip(tip2);
	label3->SetToolTip(tip3);
	label4->SetToolTip(tip4);
	label5->SetToolTip(tip5);
	label6->SetToolTip(tip6);
	
	Refresh();
}

void ModuleBasicsPage::Refresh()
{
	int protocolValue = myProtocolMenu->GetSelection();
	if (protocolValue == 1)
		myProtocol = "ntserver";
	else if (protocolValue ==2)
		myProtocol = "ext";
	else if (protocolValue == 3)
		myProtocol = "server";
	else if (protocolValue == 4)
		myProtocol = "local";
	else
		myProtocol = "pserver";

	UpdateSensitivity();

	myUsername = NeedUserName() ? myUsernameCombo->GetValue().c_str() : "";
	myServer = NeedServer() ? myServerCombo->GetValue().c_str() : "";
	myDirectory = myDirectoryCombo->GetValue();
	myModule = myModuleCombo->GetValue();

	std::string CVSROOT = CVSAction::CalcCVSRoot(myProtocol, myUsername, myServer, myDirectory);
	myCVSROOTLabel->SetLabel(CVSROOT.c_str());
}

void ModuleBasicsPage::OKClicked()
{
	WriteComboList(myServerCombo, "Checkout Server History");
	WriteComboList(myDirectoryCombo, "Checkout Directory History");
	WriteComboList(myModuleCombo, "Checkout Module History");
	WriteComboList(myUsernameCombo, "Checkout Username History");
	WriteOptionMenu(myProtocolMenu, "Checkout Protocol Default");
}

bool ModuleBasicsPage::NeedUserName()
{
	return (myProtocol == "pserver" || myProtocol == "ext" || myProtocol == "server");
}

bool ModuleBasicsPage::NeedServer()
{
	return (myProtocol != "local");
}

void ModuleBasicsPage::UpdateSensitivity()
{
	myUsernameCombo->Enable(NeedUserName());
	myServerCombo->Enable(NeedServer());
}

void ModuleBasicsPage::SetFixedModule(const std::string& module)
{
	myModuleCombo->SetValue(module.c_str());
	myModuleCombo->Enable(false);
}
