// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "ProgressDialog.h"
#include "WxwHelpers.h"
#include "WxwMain.h" // wxGetApp
#include "windows.h" // for Sleep() hackery below

ProgressDialog* ProgressDialog::ourCurrentDialog = NULL;

wxWindow* ProgressDialog::GetCurrentDialog()
{
	return static_cast<wxWindow*>(ourCurrentDialog);
}

ProgressDialogAbstract *CreateProgressDialog(bool show)
{
	ProgressDialog::ourCurrentDialog = new ProgressDialog(show);
	return ProgressDialog::ourCurrentDialog;
}

void DeleteProgressDialog(ProgressDialogAbstract * dialog)
{
	ProgressDialog::ourCurrentDialog = NULL;
	ProgressDialog* progressDialog = (static_cast<ProgressDialog*>(dialog));

	// We hide, because otherwise subsequent dialogs cause wxWindows to access violate
	// This happens, for example, with the log config dialog which can appear immediately
	// after the progress dialog.
	progressDialog->Hide(); 

	DestroyDialog(progressDialog);
}

ProgressDialog::ProgressDialog(bool show)
	: wxDialog(NULL, -1, wxString("TortoiseCVS - Working"),
	wxDefaultPosition, wxSize(600, 350), wxRESIZE_BORDER | wxDEFAULT_DIALOG_STYLE),
	myAborted(false)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	// Main box with text and button in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	myText = new wxTextCtrl(this, -1, "", wxDefaultPosition,
		wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
    myAbort = new wxButton(this, wxID_CANCEL, "Abort");

    sizerTop->Add(myText, 1, wxGROW | wxALL, 5); // 1 takes up any resize
    sizerTop->Add(myAbort, 0, wxGROW | wxALL, 5); // 0 takes no share of resize

	// Wiring
	Connect(wxID_CANCEL, wxEVT_COMMAND_BUTTON_CLICKED,
		(wxObjectEventFunction) &ProgressDialog::OnAbort);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
	Layout();

	Show(show);

	// Process events for initial refresh
	::wxYield();
}

void ProgressDialog::LookBusy()
{
	// Process events
	// TODO: What is the proper way of doing this?
	// With ThreadSafeProgress we don't need this function any more,
	// instead we want something akin to WaitForAbort?
	::wxYield();

	// Check to see if aborted and return
	if (myAborted)
		return;
}

void ProgressDialog::NewText(char *txt, int len)
{
	// Check to see if aborted and return if we have
	if (myAborted)
		return;

	// Add text in
	myText->AppendText(wxString(txt, len));
}

bool ProgressDialog::Show(bool show)
{
	return wxDialog::Show(show);
}

bool ProgressDialog::UserAborted()
{
	return myAborted;
}

void ProgressDialog::OnAbort(wxEvent& event)
{
	myAborted = true;
	Hide();
}

void ProgressDialog::WaitForAbort()
{
	if (myAborted)
		return;

	// Change "Abort" to "Close"
	myAbort->SetLabel("Close");

	// and "Working" to "Finished"
	SetTitle("TortoiseCVS - Finished");

	// Return now closes the dialog
	myAbort->SetFocus();	

	// Wait for it to close
	// TODO: This spins too much, and needs improving
	// wxWindows folk needed to advise!
	while (!myAborted)
	{
		Sleep(1);
		::wxYield();
	}
// Some attempts to improve it:
//	while(true)
//	{
//		while ( !wxGetApp().Pending() && wxGetApp().ProcessIdle() )
//			;
// a message came or no more idle processing to do
//      wxGetApp().DoMessage();
//	}
}
