// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <time.h>
#include "RevOptions.h"
#include "WxwHelpers.h"

#ifdef __BORLANDC__
	//Types required for time functions
	using std::time_t;
	using std::localtime;
#endif

RevOptions::RevOptions(wxNotebook* notebook)
	: wxPanel(notebook)
{
    wxGridSizer *gridSizer = new wxFlexGridSizer(2);
    SetAutoLayout(TRUE);
	SetSizer(gridSizer);

	myTagCheckBox = new wxCheckBox(this, -1, "Get tag/branch:");
	myTagCombo = MakeComboList(this, "Revision TagBranch History");
	gridSizer->Add(myTagCheckBox, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL | wxALL , 5);
	gridSizer->Add(myTagCombo, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL , 5);
	const char * tagTip = "Which tag or branch should be retrieved";
	myTagCheckBox->SetToolTip(tagTip);
	myTagCombo->SetToolTip(tagTip);

	const char* dateTip = 
		"Choose a date from the calendar, and/or edit the "
		"date and time in the box above the calendar.\n"
		"eg: \"1988-09-24\"  \"yesterday\"  \"24 Sep 20:05\"";
	myDateCheckBox = new wxCheckBox(this, -1, "Before date/time:");
	myDateCombo = MakeComboList(this, "Revision Date History");
	gridSizer->Add(myDateCheckBox, 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL | wxALL , 5);
	gridSizer->Add(myDateCombo, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL , 5);
	myDateCheckBox->SetToolTip(dateTip);
	myDateCombo->SetToolTip(dateTip);

	myCalendar = new wxCalendarCtrl(this, -1, wxDateTime::Now());
	wxStaticText *dummy = new wxStaticText(this, -1, "");
	gridSizer->Add(dummy, 0, wxALIGN_LEFT| wxALIGN_CENTER_VERTICAL | wxALL , 5);
	gridSizer->Add(myCalendar, 1, wxGROW | wxALIGN_CENTER_VERTICAL | wxALL , 5);
	myCalendar->SetToolTip(dateTip);
	dummy->SetToolTip(dateTip);

	// Wiring
	Connect(myTagCheckBox->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction) &RevOptions::UpdateSensitivity);
	Connect(myDateCheckBox->GetId(), wxEVT_COMMAND_CHECKBOX_CLICKED, (wxObjectEventFunction) &RevOptions::UpdateSensitivity);
	Connect(myCalendar->GetId(), wxEVT_CALENDAR_SEL_CHANGED, (wxObjectEventFunction) &RevOptions::UpdateCalendar);
	UpdateSensitivity();
	return;
}

void RevOptions::UpdateCalendar()
{
	wxDateTime date = myCalendar->GetDate();
	wxString formatted = date.Format("%d %b %Y");
	myDateCombo->SetValue(formatted);
}

void RevOptions::UpdateSensitivity()
{
	myTagCombo->Enable(myTagCheckBox->GetValue());
	myDateCombo->Enable(myDateCheckBox->GetValue());
	myCalendar->Enable(myDateCheckBox->GetValue());
}

void RevOptions::OKClicked()
{
	if (myTagCheckBox->GetValue())
		myTag = myTagCombo->GetValue();
	else
		myTag = "";

	if (myDateCheckBox->GetValue())
		myDate = myDateCombo->GetValue();
	else
		myDate = "";

	WriteComboList(myTagCombo, "Revision TagBranch History");
	WriteComboList(myDateCombo, "Revision Date History");
}
