// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "TextBinaryDialog.h"
#include "WxwHelpers.h"

bool DoTextBinaryDialog(bool& text, const std::string& file)
{
	TextBinaryDialog* textBinaryDialog = new TextBinaryDialog(file);

	int modal = textBinaryDialog->ShowModal();
	bool ret = (modal != -1);
	text = (modal == 0);
	
	DestroyDialog(textBinaryDialog);
	return ret;
}

TextBinaryDialog::TextBinaryDialog(const std::string& file)
	: wxDialog(GetAppropriateParent(), -1, wxString("TortoiseCVS - Text or Binary?"),
	wxDefaultPosition, wxSize(400,300), wxDEFAULT_DIALOG_STYLE)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	// Description of what to do
	std::string label = "Automatic detection failed - please choose whether to store\n\n" +
	  file + "\n\nas a text or a binary file.  For text files, carriage return and \nline feed "
	  "characters are automatically converted to the \nnative format of each operating system.  "
	  "Binary files are \nleft untouched.";
	wxStaticText *label1 = new wxStaticText(this, -1, label.c_str());

	// Text/Binary/Cancel button
	wxBoxSizer *sizerConfirm = new wxBoxSizer(wxHORIZONTAL);
    wxButton* text = new wxButton(this, -1, "&Text");
    wxButton* binary = new wxButton(this, -1, "&Binary");
    wxButton* cancel = new wxButton(this, wxID_CANCEL, "&Cancel");
    sizerConfirm->Add(text, 0, wxGROW | wxALL, 5);
    sizerConfirm->Add(binary, 0, wxGROW | wxALL, 5);
    sizerConfirm->Add(cancel, 0, wxGROW | wxALL, 5);

	Connect(text->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&TextBinaryDialog::TextClicked);
	Connect(binary->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&TextBinaryDialog::BinaryClicked);
	Connect(cancel->GetId(), wxEVT_COMMAND_BUTTON_CLICKED, (wxObjectEventFunction)&TextBinaryDialog::CancelClicked);

	// Main box with text and button in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	sizerTop->Add(label1, 1, wxGROW | wxALL, 5);
	sizerTop->Add(sizerConfirm, 0, wxALIGN_CENTER | wxALL, 0);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
    sizerTop->Fit(this);
	label1->SetFocus();
}

void TextBinaryDialog::TextClicked()
{
	EndModal(0);
}

void TextBinaryDialog::BinaryClicked()
{
	EndModal(1);
}

void TextBinaryDialog::CancelClicked()
{
	EndModal(-1);
}

