// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Ben Campbell
// <ben.campbell@creaturelabs.com> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "UpdateDlg.h"
#include "WxwHelpers.h"

//static
bool DoUpdateDialog( bool& resetsticky, std::string& getrevision )
{
	UpdateDlg* dlg = new UpdateDlg();
	bool ret = (dlg->ShowModal() == wxID_OK);

	if( ret )
	{
		resetsticky = dlg->myResetStickytagsButton->GetValue();
		getrevision = "";
		if( dlg->myGetRevisionButton->GetValue() )
			getrevision = dlg->myRevisionEntry->GetValue();
	}

	DestroyDialog(dlg);
	return ret;
}


UpdateDlg::UpdateDlg()
	: wxDialog(NULL, -1,
		wxString("TortoiseCVS - Update"),
		wxDefaultPosition, wxSize(400,300), wxDEFAULT_DIALOG_STYLE)
{
	SetIcon(wxIcon("TORTOISE", wxBITMAP_TYPE_ICO_RESOURCE));

	myResetStickytagsButton = new wxRadioButton( this, -1,
		"Reset sticky tags", wxDefaultPosition, wxDefaultSize, wxRB_GROUP );
	myResetStickytagsButton->SetToolTip( "Switch your local copy "
		"back to the main (HEAD) branch" );

	wxBoxSizer *sizerGetRev = new wxBoxSizer(wxHORIZONTAL);
	myGetRevisionButton = new wxRadioButton( this, -1,
		"Get Tag/Branch/Revision:", wxDefaultPosition, wxDefaultSize, 0 );
	myGetRevisionButton->SetToolTip( "Switches your working copy to "
		"a specific branch/tag/revision.\r\n"
		"Implies stickiness, so subsequent "
		"updates, commits etc. will operate on that version." );

	myResetStickytagsButton->SetValue( false);
	myGetRevisionButton->SetValue( true );
	myRevisionEntry = new wxTextCtrl(this, -1, "");
	myRevisionEntry->SetToolTip( "Branch/Tag/Revision specifier" );

	sizerGetRev->Add( myGetRevisionButton, 0, wxGROW | wxALL, 0 );
	sizerGetRev->Add( myRevisionEntry, 0, wxGROW | wxALL, 0 );
//	my...->SetFocus();

	// OK/Cancel button
	wxBoxSizer *sizerConfirm = new wxBoxSizer(wxHORIZONTAL);
    wxButton* ok = new wxButton(this, wxID_OK, "OK");
	ok->SetDefault();
    wxButton* cancel = new wxButton(this, wxID_CANCEL, "Cancel");
	sizerConfirm->Add(ok, 0, wxGROW | wxALL, 5);
    sizerConfirm->Add(cancel, 0, wxGROW | wxALL, 5);

	// Main box with everything in it
	wxBoxSizer *sizerTop = new wxBoxSizer(wxVERTICAL);
	sizerTop->Add(myResetStickytagsButton, 0, wxGROW | wxALL, 3);
	sizerTop->Add(sizerGetRev, 0, wxGROW | wxALL, 3);
	sizerTop->Add(sizerConfirm, 0, wxALIGN_CENTER | wxALL, 0);

	// Overall dialog layout settings
    SetAutoLayout(TRUE);
    SetSizer(sizerTop);
    sizerTop->SetSizeHints(this);
    sizerTop->Fit(this);

	// wire it up
	Connect( myResetStickytagsButton->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED,
		reinterpret_cast<wxObjectEventFunction>(&UpdateDlg::RefreshStuff));
	Connect( myGetRevisionButton->GetId(), wxEVT_COMMAND_RADIOBUTTON_SELECTED,
		reinterpret_cast<wxObjectEventFunction>(&UpdateDlg::RefreshStuff));

	RefreshStuff();
}

void UpdateDlg::RefreshStuff()
{
	myRevisionEntry->Enable( myGetRevisionButton->GetValue() );
}


