// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
// turn off stupid debug-too-long warning
#pragma warning(disable:4786 4503)
#endif

#include "WxwHelpers.h"
#include "ProgressDialog.h"
#include <list>
#include "../Utils/TortoiseRegistry.h"
#include <algorithm>	// for std::find

wxComboBox* MakeComboList(wxWindow* parent, const std::string& registryName, const std::string& extraDefault)
{
	// Read list from registry
	std::list< std::string > history;
	TortoiseRegistry::ReadList( registryName, history);

	// Add extra guess default (e.g. your user name on the machine)
	if (!extraDefault.empty() && std::find(history.begin(), history.end(), extraDefault) == history.end())
		history.push_back(extraDefault);

	// Make it
	std::vector<wxString> choices = ConvertToWxStringArray(history);
	std::string initialValue = choices.empty() ? "" : choices.front().c_str();
	wxComboBox* combo = new wxComboBox(parent, -1, initialValue.c_str(),
		wxDefaultPosition, wxDefaultSize, choices.size(), &choices[0]);

	// Fill in initial value
	if (!choices.empty())
	{
		// TODO: Sort this out so under NT everything doesn't look selected always
		// It's a bug in NT, I think.
	//	combo->SetValue(choices.front());
//		combo->SetSelection(0, 0);
	}

	return combo;
}

void WriteComboList(wxComboBox* widg, const std::string& registryName)
{
	std::string newValue = widg->GetValue().c_str();
	if (newValue.empty())
		return;

	// Read list from registry
	std::list< std::string > history;
	TortoiseRegistry::ReadList( registryName, history);

	// bubble up selected value
	std::list< std::string >::iterator it = std::find( history.begin(), history.end(),
		newValue );
	if( it != history.end() )
		history.erase( it );
	history.push_front( newValue );

	// drop old history items
	while( history.size() > 10 )
		history.pop_back();
	TortoiseRegistry::WriteList(registryName, history );
}

void ReadOptionMenu(wxChoice* widg, const std::string& registryName)
{
	int value = TortoiseRegistry::ReadInteger(registryName, 0);
	widg->SetSelection(value);
}

void WriteOptionMenu(wxChoice* widg, const std::string& registryName)
{
	int value = widg->GetSelection();
	TortoiseRegistry::WriteInteger(registryName, value);
}

wxWindow* GetAppropriateParent()
{
	if (ProgressDialog::GetCurrentDialog())
		return ProgressDialog::GetCurrentDialog();

	return NULL;
}

void DestroyDialog(wxDialog* dlg)
{
	delete dlg;

	// This causes problems with latest CVS wxWindows
	// (when a second dialog is shown, while the first one
	// is being deleted in the garbage collector, the second
	// window becomes mysteriously hidden)
	// Really, I think Destroy() is the right thing to call
	// though?
	// dlg->Destroy();
}

