/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "PropSheetLogParser.h"
#include "../Utils/PropSheetMsg.h"

//********************************************************************************************
// Startup and shutdown

PropSheetLogParser::PropSheetLogParser(FILE *_input, HWND _output) :
	input(_input), output(_output)
{
	symbolic.SetNotify(*this, 0, &PropSheetLogParser::NotifyEndSymbolic);
	revFile.SetNotify(*this, 0, &PropSheetLogParser::NotifyEndRevFile);
	thread.BeginThread(this, &PropSheetLogParser::ThreadMain);
}

//********************************************************************************************
// Parse actions

void PropSheetLogParser::Reset()
{
	RcsFile().Reset();
	RevFile().Reset();
	Branch().Reset();
	Symbolic().Reset();
	Lock().Reset();
	Access().Reset();
}

void PropSheetLogParser::NotifyEndSymbolic(CRevNumber & _symbolic)
{
	PropSheetSymbolicMsg data;
	data.revisionNumber = _symbolic.str();
	data.tag = _symbolic.Tag();
	data.isBranch = _symbolic.isbranch();

	PropSheetMsgChannel sender;
	data.Serialize(sender);
	sender.Send(output, NULL);
}

void PropSheetLogParser::NotifyEndRevFile(CRevFile &_revFile)
{
	PropSheetRevisionMsg data;
	data.revisionNumber = _revFile.RevNum().str();
	data.author = _revFile.Author();
	data.timestamp = _revFile.RevTime();

	PropSheetMsgChannel sender;
	data.Serialize(sender);
	sender.Send(output, NULL);
}

//********************************************************************************************
// Main parsing thread

void PropSheetLogParser::ThreadMain()
{
	gLog = this;
	yycvsin = input;
	yycvsparse();
}
