/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef  PropSheetLogParser_H
#define PropSheetLogParser_H

#include <windows.h>
#include <stdio.h>

#include "../cvstree/CvsLogParser.h"
#include "../cvstree/common.h"
#include "../Utils/Thread.h"

class PropSheetLogParser;

template <class PARSED>
class PropSheetLogParserThrowaway : public Parser<PARSED>
{
public:
	PropSheetLogParserThrowaway() : isParsing(0), parent(0), notifyStartPtr(0), notifyEndPtr(0) {}
	virtual ~PropSheetLogParserThrowaway() {}
	
	virtual void StartParse() { theOne = PARSED(); isParsing= true; NotifyStart(**this); }
	virtual void EndParse() { NotifyEnd(**this); isParsing= false; }
	virtual bool IsParsing() { return isParsing; }
	virtual PARSED & operator*() { return theOne; }
	virtual PARSED * operator->() { return &theOne; }
	virtual void Reset() { theOne = PARSED(); }

	typedef void (PropSheetLogParser::*NotifyFuncPtr)(PARSED & _notify);

	virtual void SetNotify(PropSheetLogParser & _parent, NotifyFuncPtr _notifyStartPtr = 0, NotifyFuncPtr _notifyEndPtr = 0)
	{
		parent = &_parent;
		notifyStartPtr = _notifyStartPtr;
		notifyEndPtr = _notifyEndPtr;
	}

protected:
	inline void NotifyStart(PARSED & _notify) { if (notifyStartPtr  != 0) (parent->*notifyStartPtr)(_notify); }
	inline void NotifyEnd(PARSED & _notify) { if (notifyEndPtr  != 0) (parent->*notifyEndPtr)(_notify); }

	bool isParsing;
	PARSED theOne;

	PropSheetLogParser * parent;
	NotifyFuncPtr notifyStartPtr;
	NotifyFuncPtr notifyEndPtr;
};

class PropSheetLogParser : public CvsLogParser
{
public:
	PropSheetLogParser(FILE *input, HWND output);
	virtual ~PropSheetLogParser() {}

	virtual void Reset();
	virtual Parser<CRcsFile> & RcsFile() { return rcsFile; }
	virtual Parser<CRevFile> & RevFile() { return revFile; }
	virtual Parser<CRevNumber> & Branch() { return branch; }
	virtual Parser<CRevNumber> & Symbolic() { return symbolic; }
	virtual Parser<CRevNumber> & Lock() { return lock; }
	virtual Parser<CLogStr> & Access() { return access; }

protected:
	// where to read the CVS log output from
	FILE * input;
	HWND output;

	// thread to collect and parse the CVS log output
	Thread<PropSheetLogParser> thread;
	virtual void ThreadMain();

	// one of each of the parsed objects
	PropSheetLogParserThrowaway<CRcsFile> rcsFile;
	PropSheetLogParserThrowaway<CRevFile> revFile;
	PropSheetLogParserThrowaway<CRevNumber> branch;
	PropSheetLogParserThrowaway<CRevNumber> symbolic;
	PropSheetLogParserThrowaway<CRevNumber> lock;
	PropSheetLogParserThrowaway<CLogStr> access;

	// functions to send the messages to TortoiseShell
	virtual void NotifyEndSymbolic(CRevNumber & _symbolic);
	virtual void NotifyEndRevFile(CRevFile &_revFile);
};

#endif
