// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - January 2001

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include "ThreadSafeProgress.h"

ThreadSafeProgress::ThreadSafeProgress(ProgressDialogAbstract* forwardTo)
{
	myForwardTo = forwardTo;
	myWaitForAbort = false;
	myFinished = false;
}

ThreadSafeProgress::~ThreadSafeProgress()
{
}

void ThreadSafeProgress::LookBusy()
{
	// Don't need to store these up, as we call them anyway in the main thread
}

void ThreadSafeProgress::NewText(char *txt, int len)
{
	SimpleMutexLock lock(myMutex);
	std::string newText(txt, len);
	myNewText += newText;
}

bool ThreadSafeProgress::UserAborted()
{
	SimpleMutexLock lock(myMutex);
	return myForwardTo->UserAborted();
}

void ThreadSafeProgress::WaitForAbort()
{
	SimpleMutexLock lock(myMutex);
	myWaitForAbort = true;
}

bool ThreadSafeProgress::Show(bool show)
{
	SimpleMutexLock lock(myMutex);
	return myForwardTo->Show(show);
}

void ThreadSafeProgress::Main()
{
	myFinished = false;
	while(!myFinished)
	{
		Sleep(2);
		myForwardTo->LookBusy();

		{
			SimpleMutexLock lock(myMutex);

			if (myWaitForAbort)
			{
				myWaitForAbort = false;
				myForwardTo->WaitForAbort();
			}

			if (!myNewText.empty())
			{
				myForwardTo->NewText(&myNewText[0], myNewText.size());
				myNewText = "";
			}
		}
	}
}


void ThreadSafeProgress::Finished()
{
	SimpleMutexLock lock(myMutex);
	myFinished = true;
}

