// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#if defined(__BORLANDC__)
#include <condefs.h>
USEUNIT("..\DialogsWxw\BranchTagDlg.cpp");
USEUNIT("..\DialogsWxw\CheckoutDialog.cpp");
USEUNIT("..\DialogsWxw\ChooseFile.cpp");
USEUNIT("..\DialogsWxw\CommitDialog.cpp");
USEUNIT("..\DialogsWxw\LogConfigDialog.cpp");
USEUNIT("..\DialogsWxw\LoginDialog.cpp");
USEUNIT("..\DialogsWxw\MakeModuleDialog.cpp");
USEUNIT("..\DialogsWxw\MergeDlg.cpp");
USEUNIT("..\DialogsWxw\MessageDialog.cpp");
USEUNIT("..\DialogsWxw\ModuleBasicsPage.cpp");
USEUNIT("..\DialogsWxw\PreferencesDialog.cpp");
USEUNIT("..\DialogsWxw\ProgressDialog.cpp");
USEUNIT("..\DialogsWxw\RevOptions.cpp");
USEUNIT("..\DialogsWxw\TextBinaryDialog.cpp");
USEUNIT("..\DialogsWxw\UpdateDlg.cpp");
USEUNIT("..\DialogsWxw\WxwHelpers.cpp");
USEUNIT("..\DialogsWxw\WxwMain.cpp");
USEUNIT("..\CVSGlue\CVSAction.cpp");
USEUNIT("..\CVSGlue\CvsEntries.cpp");
USEUNIT("..\CVSGlue\CVSStatus.cpp");
USEUNIT("..\CVSGlue\FileTraversal.cpp");
USEUNIT("..\CVSGlue\getline.c");
USEUNIT("..\CVSGlue\MakeArgs.cpp");
USEUNIT("..\CVSGlue\ndir.c");
USEUNIT("..\CVSGlue\PreserveChdir.cpp");
USEUNIT("..\CVSGlue\TextBinary.cpp");
USEUNIT("..\CVSGlue\ustr.cpp");
USEUNIT("..\Utils\HttpSnaffle.cpp");
USEUNIT("..\Utils\Keyboard.cpp");
USEUNIT("..\Utils\PathUtils.cpp");
USEUNIT("..\Utils\Preference.cpp");
USEUNIT("..\Utils\TortoiseRegistry.cpp");
USEUNIT("..\Utils\TortoiseUtils.cpp");
USEUNIT("ThreadSafeProgress.cpp");
USEUNIT("TortoisePreferences.cpp");
USEUNIT("TortoiseTip.cpp");
USEUNIT("WebLog.cpp");
USERC("..\DialogsWxw\TortoiseWxw.rc");
USEFILE("vctobpr.log");
USELIB("..\..\..\wx2\src\msw\Release\wxvc.lib");
USEUNIT("..\Utils\TortoiseException.cpp");
USEUNIT("..\Utils\ShellUtils.cpp");
USEUNIT("PropSheetLogParser.cpp");
USEUNIT("..\cvstree\parser.cpp");
USEUNIT("..\cvstree\lexer.cpp");
USEUNIT("..\cvstree\CvsLog.cpp");
USEUNIT("..\Utils\Serialize.cpp");
USELIB("C:\Program Files\National Instruments\cvi\sdk\lib\WinInet.Lib");
USEUNIT("..\Utils\PropSheetMsg.cpp");
//---------------------------------------------------------------------------
#endif // defined(__BORLANDC__)

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

// The TortoiseAct program takes as parameters a command name (myVerb)
// and a list of files.  It then performs that TortoiseCVS action, including 
// any user interface necessary.

// The parameters can be passed by file rather than in command line
// arguments, with the command line pointing to the file. TortoiseShell dll
// calls TortoiseAct everytime a context menu is selected.

// The TortoiseAct class contains the guts of the program.

// The actual application starts in DialogsWxw/WxwMain, which simply 
// instantiates this class (TortoiseAct) and calls some simple functions in it.

#include "../Utils/TortoiseDebug.h"

#include <string>
#include <vector>
#include <windows.h>
#include <fstream>
#include <algorithm>

#include "TortoiseAct.h"
#include "TortoiseDialogs.h"
#include "TortoisePreferences.h"
#include "WebLog.h"
#include "PropSheetLogParser.h"

#include "../CVSGlue/CVSAction.h"
#include "../CVSGlue/CVSStatus.h"
#include "../Utils/PathUtils.h"
#include "../Utils/TortoiseUtils.h"
#include "../Utils/TortoiseRegistry.h"
#include "../Utils/TortoiseException.h"
#include "../Utils/Keyboard.h"
#include "../Utils/ShellUtils.h"

TortoiseAct::TortoiseAct() :
  myRemoteHandle(0)
{
	{
		// TODO: Write this a more explicit or better way
		// Force flushing of preferences to registry, so all values
		// exist, and defaults are as specified in TortoisePreferences.
		TortoisePreferences prefs;
	}

	// Work out a good home directory...
	std::string home = TortoiseRegistry::ReadString("HOME");
	// We usually recalculate home every time - so ignore the
	// value in the registry.  The user can turn that off though!
	if (TortoiseRegistry::ReadBoolean("Always Recalculate Home", true))
		home = ""; 
	if (home.empty() || !IsDirectory(home.c_str()))
	{
		// Try home environment variable
		char *got = getenv("HOME");
		home = got ? got : "";
	}
	if (home.empty() || !IsDirectory(home.c_str()))
	{
		// Try WinCVS's home directory
		home = TortoiseRegistry::ReadWinCVSString("P_Home");
	}
	if (home.empty() || !IsDirectory(home.c_str()))
	{
		// Try My Documents
		home = MyDocumentsFolder();
	}
	if (home.empty() || !IsDirectory(home.c_str()))
	{
		// Default to somewhere that must exist!
		home = "C:";
	}
	// Write out our nice home directory
	TortoiseRegistry::WriteString("HOME", home);
}

// Add new verbs here, and in DefaultMenus.cpp in TortoiseShell.
// The same verbs are Explorer verbs, so there is the possibility of
// calling TortoiseCVS via DDE or OLE automation upon Explorer, but I've
// never tried it.  That's why they all begin with "CVS".
void TortoiseAct::PerformAction()
{
	if (myVerb == "CVSAdd")
		PerformAddMenu();
	else if (myVerb == "CVSAddRecursive")
		PerformAddMenuRecursive();
	else if (myVerb == "CVSCheckOut" || myVerb == "CVSCheckout")
		PerformCheckoutMenu();
	else if (myVerb == "CVSTortoiseCode")
		PerformTortoiseCheckout();
	else if (myVerb == "CVSwxWindowsCode")
		PerformwxWindowsCheckout();
	else if (myVerb == "CVSMakeModule")
		PerformMakeModuleMenu();
	else if (myVerb == "CVSCommit")
		PerformCommitMenu();
	else if (myVerb == "CVSCommitDialog")
		PerformCommitDialogMenu();
	else if (myVerb == "CVSDiff")
		PerformDiffMenu();
	else if (myVerb == "CVSMakePatch")
		PerformMakePatchMenu();
	else if (myVerb == "CVSLog")
		PerformLogMenu();
	else if (myVerb == "CVSLogPropSheet")
		PerformLogPropSheet();
	else if (myVerb == "CVSRemove")
		PerformRemoveMenu();
	else if (myVerb == "CVSUpdate")
		PerformUpdateMenu();
	else if (myVerb == "CVSWebLog")
		PerformWebLogMenu();
	else if (myVerb == "CVSTag")
		PerformTagMenu();
	else if (myVerb == "CVSBranch")
		PerformBranchMenu();
	else if (myVerb == "CVSMerge")
		PerformMergeMenu();
	else if (myVerb == "CVSHelp")
		PerformHelp();
	else if (myVerb == "CVSPrefs")
		PerformPrefs();
	else
		DoMessageDialog("Verb not known:\n" + myVerb);
}

// Stores arguments inside the class.
// Returns true if args seem valid.
bool TortoiseAct::SussArgs( int ac, char* av[])
{
	if( ac < 2 )
	{
		DoMessageDialog("Too few args passed to TortoiseAct.\n\n"
			"TortoiseAct is designed to be run by the Explorer extension TortoiseShell.dll. "
			"You don't need to run it yourself unless you have a special purpose.\n\n"
			"Command line arguments:\n"
			"TortoiseAct cvs-verb [-h handle] [-f file-list] [files...]");
		return false;
	}

	// 1st arg must be verb
	myVerb = av[1];

	int i=2;
	while( i<ac )
	{
		if( !stricmp( "-h", av[i] ) )
		{
			bool valid = true;
			if( ++i < ac )
			{
				long myRemoteHandleLong;
				valid = (sscanf(av[i], "0x%lx", &myRemoteHandleLong) == 1);
				myRemoteHandle = (HWND)  myRemoteHandleLong;
			}
			else
				valid = false;

			if (!valid)
			{
				DoMessageDialog("Invalid args passed to "
					"TortoiseAct (expected handle \"-h 0xXXXXXXXX\")");
				return false;
			}
		}
		else
		if( !stricmp( "-f", av[i] ) )
		{
			++i;
			if( i>=ac )
			{
				DoMessageDialog("Invalid args passed to "
					"TortoiseAct (expected a filename after '-f')");
				return false;
			}
			// Read filenames from a file...
			{
				std::ifstream in( av[i] );
				if (!in.good())
				{
					std::string message = "File containing input filenames not found:\n";
					message += av[i];
					DoMessageDialog(message);
					return false;
				}

				// A file on each line...
				while(true)
				{
					std::string file;
					std::getline(in, file);
					if (file.empty())
						break;
					myAllFiles.push_back(file);
				}
			}

			// Remove the file
			if (av[i] != "c:testinput.txt") // debugging
				DeleteFile( av[i] );
		}
		else
		{
			// any other args considered filenames.
			myAllFiles.push_back( av[i] );
		}
		++i;
	}

	return PreprocessFileList();
}

// These two act as a pair - one finds a relative filename,
// the other returns the directory that it is relative to.

// The clever thing is what happens to directories:  For
// top level ones, we have to work inside the directory and
// return "." as the file.  For inner directories, we are
// better returning the directory above as the directory,
// and ourselves as a file.  This is because of the way
// CVS processes command lines.

std::string TortoiseAct::FindRelativeName(std::string file, bool fancyRelativeNames)
{
	// Disabled for recursive adding
	// (since then we are deep scanning a folder which is not yet in 
	// CVS, so it the rest of this function would get confused and think
	// we were at the top level when we weren't)
	if (!fancyRelativeNames)
		return ExtractLastPart(file);

	// Case when we begin at the top level and we are
	// a directory, so we need to return ourself as "."
	if (IsDirectory(file.c_str()) && !CVSFolderHere(GetDirectoryAbove(file)))
		return ".";

	// Otherwise scan for that level
	std::string lastPart = ExtractLastPart(file);
	std::string directory = GetDirectoryAbove(file);
	while (CVSFolderHere(GetDirectoryAbove(directory)))
	{
		lastPart = ExtractLastPart(directory) + "\\" + lastPart;
		directory = GetDirectoryAbove(directory);
	}
	return lastPart;
}

std::string TortoiseAct::FindDirectoryName(std::string file, bool fancyRelativeNames)
{
	// Disabled for recursive adding
	// (see in FindRelativeName above for why)
	if (!fancyRelativeNames)
		return GetDirectoryAbove(file);

	// Scan for the top level CVS directory
	// - we do everything relative to that
	std::string directory = GetDirectoryPart(file);
	while (CVSFolderHere(GetDirectoryAbove(directory)))
	{
		std::string prevdir = directory;
		directory = GetDirectoryAbove(directory);
		if (directory == prevdir)
		{
			TortoiseFatalError("You appear to have a folder called \"CVS\"\n"
							"in a top level directory.  Sorry, but\n"
							"you cannot call a folder which contains\n"
							"CVS checkouts \"CVS\", as that folder name\n"
							"is used internally by CVS.");
		}
	}

	RemoveTrailingDelimiter(directory);
	return directory;
}

bool TortoiseAct::VerbNeedsFiles()
{
	return !(myVerb == "CVSHelp" || myVerb == "CVSPrefs");
}

// Sort into groups of files which are in the same sandbox.  Each cluster
// can then be passed into CVS together.  Files from different directories
// can be operated on at once from the Find In Files dialog in Explorer.
bool TortoiseAct::PreprocessFileList(bool fancyRelativeNames)
{
	// The main verbs depend on there being at least one file
	if (VerbNeedsFiles() && myAllFiles.empty())
	{
		DoMessageDialog("Empty file list disallowed");
		return false;
	}

	// Break into chunks of the same checked out sandbox 
	// For each directory, store a vector of files and other data in the
	// DirectoryGroup.  The group has relative and absolute filenames
	// because CVS requires relative, but it is convenient for TortoiseCVS
	// to use the absolute ones when it needs to.
	// (to be precise, only NT server _requires_ relative paths to a
	// current directory.  Unix CVS server can cope with absolute ones.
	// However, it is good practice anyway.)
	std::map<std::string, DirectoryGroup > directoryMap;
	for (unsigned int i = 0; i < myAllFiles.size(); ++i)
	{
		std::string file = myAllFiles[i];

		// Find directory
		std::string directory = FindDirectoryName(file, fancyRelativeNames);

		// Find relative filename
		std::string relativeFile = FindRelativeName(file, fancyRelativeNames);

		// Put in map
		DirectoryGroup& group = directoryMap[directory];
		group.myDirectory = directory;
		group.myAbsoluteFiles.push_back(file);
		group.myRelativeFiles.push_back(relativeFile);
	}

	// Copy into the vector for easier access by looping
	myDirectoryGroups.clear();
	std::map<std::string, DirectoryGroup >::iterator it;
	for (it = directoryMap.begin(); it != directoryMap.end(); ++it)
		myDirectoryGroups.push_back(it->second);

	// Cache single values, so commands like Diff can look them up easily
	if (myAllFiles.size() == 1)
	{
		mySingleAbsolute = myAllFiles[0];
		mySingleDirectory = FindDirectoryName(mySingleAbsolute, fancyRelativeNames);
		mySingleRelative = FindRelativeName(mySingleAbsolute, fancyRelativeNames);
	}

	return true;
}


///////////////////////////////////////////////////////////////////////////

void TortoiseAct::PerformAddMenu()
{
	CVSAction glue;

	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		// Sort into text/binary first, and add in two clumps,
		// meaning fewer CVS commands, so quicker.
		std::vector<std::string> binFiles;
		std::vector<std::string> textFiles;
		for (unsigned int i = 0; i < group.size(); ++i)
		{
			std::string absoluteFile = group.myAbsoluteFiles[i];

			bool bin = false;
			if (!IsDirectory(absoluteFile.c_str()))
			{
				bin = CVSStatus::IsFileBinary(absoluteFile);
				bool text = CVSStatus::IsFileText(absoluteFile);
				if (bin == text)
				{
					bool textSelected;
					bool result = DoTextBinaryDialog(textSelected, absoluteFile);
					if (!result)
					{
						glue.SetCloseIfOK(true);
						return;
					}
					text = textSelected;
					bin = !textSelected;
				}
				ASSERT(bin != text);
			}

			CVSStatus::FileStatus status = CVSStatus::GetFileStatus(absoluteFile);
			if (status == CVSStatus::STATUS_NOTINCVS || status == CVSStatus::STATUS_NOWHERENEAR_CVS)
			{
				if (bin)
					binFiles.push_back(group.myRelativeFiles[i]);
				else
					textFiles.push_back(group.myRelativeFiles[i]);
			}
		}

		// Add text files and directories
		if (!textFiles.empty())
		{
			MakeArgs args;
			args.add("add");
			for (unsigned int i = 0; i < textFiles.size(); ++i)
				args.add(textFiles[i]);
			glue.Command(group.myDirectory, args);
		}

		// Add binary files
		if (!binFiles.empty())
		{
			MakeArgs args;
			args.add("add");
			args.add("-kb");
			for (unsigned int i = 0; i < binFiles.size(); ++i)
				args.add(binFiles[i]);
			glue.Command(group.myDirectory, args);
		}	
	}

	if (glue.AtLeastOneSuccess())
		RefreshExplorerWindow();
}

// TODO: Make this ignore .cvsignore and CVSROOT/cvsignore
// files, and not add them.
void TortoiseAct::PerformAddMenuRecursive()
{
	// Get all files recursively
	std::vector<std::string> files;
	for (unsigned int i = 0; i < myAllFiles.size(); ++i)
	{
		if (IsDirectory(myAllFiles[i].c_str()))
		{
			files.push_back(myAllFiles[i]);
			GetRecursiveContents(myAllFiles[i], files, "*");
		}
	}
	myAllFiles = files;

	// Redo the preprocessing
	if (PreprocessFileList(false))
	{
		// If OK, add all those files
		PerformAddMenu();
	}
}

void TortoiseAct::PerformCheckoutMenu()
{
	// Only allow one directory
	if (mySingleAbsolute.empty())
		return;
	if (!IsDirectory(mySingleAbsolute.c_str()))
		return;

	std::string protocol, userName, serverMachine, repositoryDirectory, module, tag, date;
	if (DoCheckoutDialog(protocol, userName, serverMachine, repositoryDirectory, module, tag, date ))
	{
		CVSAction glue;
		glue.SetCVSRoot(protocol, userName, serverMachine, repositoryDirectory);
		{
			MakeArgs args;
			args.add("checkout");
			if( !tag.empty() )
			{
				args.add("-r");
				args.add( tag );
			}
			if( !date.empty() )
			{
				args.add("-D");
				args.add( date );
			}

			args.add(module);
			glue.Command(mySingleAbsolute, args);
		}
	}
}

void TortoiseAct::PerformTortoiseCheckout()
{
	std::string project = "cvsgui";
	std::string module = "TortoiseCVS";

	// Only allow one directory
	if (mySingleAbsolute.empty())
		return;
	if (!IsDirectory(mySingleAbsolute.c_str()))
		return;

	CVSAction glue;
	glue.SetCVSRoot("pserver", "anonymous", "cvs." + project + ".sourceforge.net", "/cvsroot/" + project);
	// Password is empty string for Sourceforge
	{
		MakeArgs args;
		args.add("checkout");
		args.add(module);
		glue.Command(mySingleAbsolute, args);
	}

	std::string message = "You can now compile the TortoiseCVS code.  It will use a\n"
						  "precompiled set of wxWindows libraries unless you also \n"
						  "fetch the wxWindows code.";
	message += "\n\nCreate a patch file: If you fix some bugs or add some new\n"
		       "features, please submit the changes for other users.  Right click\n"
			   "on the changed files or folders and choose Make Patch from the\n"
			   "CVS submenu.";
	glue.GetProgressDialog()->NewText((char*)message.c_str(), message.size());
}

void TortoiseAct::PerformwxWindowsCheckout()
{
	// Only allow one directory
	if (mySingleAbsolute.empty())
		return;
	if (!IsDirectory(mySingleAbsolute.c_str()))
		return;

	CVSAction glue;
	glue.SetCVSRoot("pserver", "anoncvs", "cvs.wxwindows.org", "/home/wxcvs");
	glue.SetPassword("anoncvs");
	{
		MakeArgs args;
		args.add("login");
		if (!glue.Command(mySingleAbsolute, args))
			return;
	}
	glue.SetCVSRoot("pserver", "anoncvs", "cvs.wxwindows.org", "/home/wxcvs");
	{
		MakeArgs args;
		args.add("checkout");
		args.add("wxWindows");
		if (!glue.Command(mySingleAbsolute, args))
			return;
	}

	// See if we need to make the setup file
	std::string setupDir = mySingleAbsolute + "\\wxWindows\\include\\wx\\msw\\";
	std::string setupSource = setupDir + "setup0.h";
	std::string setupDest = setupDir + "setup.h";
	if (!FileExists(setupSource.c_str()))
	{
		std::string message = "No template setup file to copy from: " + setupSource + "\n";
		glue.GetProgressDialog()->NewText((char*)message.c_str(), message.size());
		return;
	}
	if (FileExists(setupDest.c_str()))
	{
		std::string message = "Patched setup file already exists: " + setupDest + "\n";
		glue.GetProgressDialog()->NewText((char*)message.c_str(), message.size());
		return;
	}

	// Do it, with patching
	std::string message = "Patching setup file...\n";
	glue.GetProgressDialog()->NewText((char*)message.c_str(), message.size());

	int patchCount = 0;
	std::ifstream in(setupSource.c_str());
	std::ofstream out(setupDest.c_str());
	out << "// TortoiseCVS copied and patched this file\n";
	out << "// See comments below for details.\n\n";
	while (!in.eof())
	{
		std::string line;
		std::getline(in, line);
		if (line == "#define wxUSE_DEBUG_NEW_ALWAYS 1")
		{
			out << "\n// Patched by TortoiseCVS:\n";
			out << "// " << line << "\n";
			out << "#define wxUSE_DEBUG_NEW_ALWAYS 0\n";
			patchCount++;
		}
		else if (line == "#define wxUSE_MEMORY_TRACING 1")
		{
			out << "\n// Patched by TortoiseCVS:\n";
			out << "// " << line << "\n";
			out << "#define wxUSE_MEMORY_TRACING  0\n";
			patchCount++;
		}
		else if (line == "#define wxUSE_GLOBAL_MEMORY_OPERATORS 1")
		{
			out << "\n// Patched by TortoiseCVS:\n";
			out << "// " << line << "\n";
			out << "#define wxUSE_GLOBAL_MEMORY_OPERATORS 0\n";
			patchCount++;
		}
		else
		{
			out << line << "\n";
		}
	}
	if (patchCount == 3)
		message = "Everything complete OK.";
	else
		message = "Warning: Exactly three patches were not applied.";
	message += "\n\nYou can now compile the wxWindows code.  If the TortoiseCVS\n"
				"code folder is within the same folder as the wxWindows code,\n"
				"the TortoiseCVS Visual C++ project will automatically find the\n"
				"wxWindows one.";
	message += "\n\nCreate a patch file: If you fix some bugs or add some new\n"
		       "features, please submit the changes for other users.  Right click\n"
			   "on the changed files or folders and choose Make Patch from the\n"
			   "CVS submenu.";
	glue.GetProgressDialog()->NewText((char*)message.c_str(), message.size());
}

// Not your normal CVS import - creatres just the top level folder,
// and does it "in-place" (or at least gives the illusion of doing so).
void TortoiseAct::PerformMakeModuleMenu()
{
	// Only allow one directory, which isn't in CVS already
	if (mySingleAbsolute.empty())
		return;
	if (!IsDirectory(mySingleAbsolute.c_str()))
		return;
	if (CVSFolderHere(mySingleAbsolute))
		return;

	// Fetch parameters
	std::string module = ExtractLastPart(mySingleAbsolute);

	std::string protocol, userName, serverMachine, repositoryDirectory;
	if (!DoMakeModuleDialog(protocol, userName, serverMachine, repositoryDirectory, module))
		return;
	
	// Get temporary folder to work in
	std::string tempFolder = GetTemporaryPath();
	EnsureTrailingDelimiter(tempFolder);
	tempFolder += "TortoiseCVS make new module temp";
	EnsureTrailingDelimiter(tempFolder);
	CreateDirectory(tempFolder.c_str(), NULL);

	// Create empty directory in it (named after module)
	CreateDirectory((tempFolder + module).c_str(), NULL);

	// Import that directory
	bool ok;
	CVSAction glue;
	glue.SetCVSRoot(protocol, userName, serverMachine, repositoryDirectory);
	{
		MakeArgs args;
		args.add("import");
		args.add("-m");
		args.add(""); // empty comment (never seen as we don't import files)
		args.add(module);
		args.add("tcvs-vendor");
		args.add("tcvs-release");
		ok = glue.Command(tempFolder, args);
	}

	// Now we can remove it, and the outer one
	RemoveDirectory((tempFolder + module).c_str());
	RemoveDirectory(tempFolder.c_str());

	// Give up, if we failed the import
	if (!ok)
		return;

	// Then check it out again in the temporary directory
	// (we can't just check on top of the original, unfortunately)
	glue.SetCVSRoot(protocol, userName, serverMachine, repositoryDirectory);
	{
		MakeArgs args;
		args.add("checkout");
		args.add(module);
		if (!glue.Command(tempFolder, args))
			return;
	}

	// And copy CVS part magically into place
	if (!CopyFolderRecursively(tempFolder + module + "\\CVS", mySingleAbsolute))
	{
		std::string message = "Failed to copy checked out version into place";
		MessageBox(0, message.c_str(), "TortoiseCVS", MB_OK);
	}

	RefreshExplorerWindow();
}

// Original commit, which just uses the default CVS behaviour of 
// launching a text editor for the comment (usually Notepad under Windows)
void TortoiseAct::PerformCommitMenu()
{
	CVSAction glue;

	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("commit");
		for (unsigned int i = 0; i < group.size(); ++i)
			args.add(group.myRelativeFiles[i]);
		glue.Command(group.myDirectory, args);
	}

	if (FileSelected() && glue.AtLeastOneSuccess())
		RefreshExplorerWindow();
}

// Improved commit, with dialogs
void TortoiseAct::PerformCommitDialogMenu()
{
	// Get a list of all the modified/added/removed files
	std::vector<std::string> modified, added, removed;
	for (unsigned int i = 0; i < myAllFiles.size(); ++i)
		CVSStatus::RecursiveScan(myAllFiles[i], modified, added, removed);

	// Sort them into order
	std::sort(modified.begin(), modified.end());
	std::sort(added.begin(), added.end());
	std::sort(removed.begin(), removed.end());

	// Check there are some files
	if (modified.empty() && added.empty() && removed.empty())
	{
		DoMessageDialog("There is nothing to commit.\n\n"
						"All the files and folders that you\n"
						"selected are unchanged since you\n"
						"last performed a commit.");
		return;
	}

	// Retrieve a comment about them
	std::string comment = TortoiseRegistry::ReadString("FailedCommitMessage");;
	std::vector<std::string> userSelection;
	bool result = DoCommitDialog(modified, added, removed, comment, userSelection);
	if (!result || userSelection.empty())
		return;

	// Redo the preprocessing into groups
	myAllFiles = userSelection;
	if (!PreprocessFileList())
	{
		std::string message = "Failed to reprocess file list";
		MessageBox(0, message.c_str(), "TortoiseCVS", MB_OK);
		// TODO: Better error code everywhere
		return;
	}

	// Now commit all the files in groups (by repository)
	CVSAction glue;
	bool ok = true;
	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("commit");
		args.add("-m");
		args.add(comment);
		for (unsigned int i = 0; i < group.size(); ++i)
			args.add(group.myRelativeFiles[i]);
	
		if (!glue.Command(group.myDirectory, args))
			ok = false;
	}

	// Remember comment from failed commit
	if (!ok)
		TortoiseRegistry::WriteString("FailedCommitMessage", comment);
	else
		TortoiseRegistry::WriteString("FailedCommitMessage", "");

	// Clear the Explorer window (unless it was all directories being
	// updated, in which case no point)
	if (FileSelected() && glue.AtLeastOneSuccess())
		RefreshExplorerWindow();
}


void TortoiseAct::PerformDiffMenu()
{
	// Only allow one file
	if (mySingleAbsolute.empty())
		return;
	if (IsDirectory(mySingleAbsolute.c_str()))
		return;

	// Forget the diff program if the user holds down Control,
	// so they can choose a new diff program
	bool forceQuery = false;
	if (IsControlDown())
		forceQuery = true;

	std::string diffFile = GetTemporaryPath();
	diffFile += "tortoise.old." + ExtractLastPart(mySingleAbsolute);
	DeleteFile(diffFile.c_str());


	{
		CVSAction glue;
		glue.SetStreamFile(diffFile);
		glue.SetCloseIfOK(true);
		MakeArgs args;
		args.add("update");
		args.add("-p");
		args.add(mySingleRelative);

		if (!glue.Command(mySingleDirectory, args))
			return;
	}

	if (!FileExists(diffFile.c_str()))
	{
		DoMessageDialog("This file is new and has never been committed to\n"
						"the server or is an empty file on the server.\n\n"
						+ mySingleAbsolute);
		return;
	}
	
	bool again = false;
	do
	{
		std::string externalApp = TortoiseRegistry::ReadString("External Diff Application");
		// First time round, try WinCVS's external diff as well
		// (we only try the first time, in case it is an invalid path)
		if (!again && externalApp.empty())
			externalApp = TortoiseRegistry::ReadWinCVSString("P_Extdiff");
		again = false;
		if (externalApp.empty() || forceQuery)
			externalApp = DoOpenFileDialog("Choose external diff application", externalApp, "Executables (*.exe)|*.exe");
		if (!externalApp.empty())
		{
			// Perform diff, waiting for it to finish (so we can delete file)
			std::string command = "\"" + externalApp + "\" \"" + diffFile + "\" \"" + mySingleAbsolute + "\"";
			if (!LaunchCommand(command, true))
			{
				std::string message = "Failed to launch external diff\n" + command;
				DoMessageDialog(message);
				externalApp = "";
				again = true;
			}

			// Clean up
			DeleteFile(diffFile.c_str());
			TortoiseRegistry::WriteString("External Diff Application", externalApp);
		}
		else
		{
			// Peform a textual diff
			CVSAction glue;
			MakeArgs args2;
			args2.add("diff");
			args2.add(mySingleRelative);
			glue.Command(mySingleDirectory, args2);	
		}
	}
	while (again);
}

void TortoiseAct::PerformLogMenu()
{
	CVSAction glue;

	glue.SetCloseIfOK(false);
	
	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("log");
		for (unsigned int i = 0; i < group.size(); ++i)
		{
			CVSStatus::FileStatus status = CVSStatus::GetFileStatus(group.myAbsoluteFiles[i]);
			if( status == CVSStatus::STATUS_INCVS ||
				status == CVSStatus::STATUS_CHANGED ||
				status == CVSStatus::STATUS_CONFLICT )
			{
				args.add(group.myRelativeFiles[i]);
			}
		}
		glue.Command(group.myDirectory, args);
	}
}

bool IsFileRemovable(const std::string& file)
{
	CVSStatus::FileStatus status = CVSStatus::GetFileStatus(file);
	if (status != CVSStatus::STATUS_NOTINCVS && status != CVSStatus::STATUS_NOWHERENEAR_CVS &&
		status != CVSStatus::STATUS_OUTERDIRECTORY)
	{
		// Can't yet directly remove folders with other stuff under them
		if (!CVSStatus::InCVSHasSubdirs(file))
			return true;
	}
	return false;
}

void TortoiseAct::PerformRemoveMenu()
{
	CVSAction glue;

	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("remove");
		unsigned int i;

		// Construct command, and move all the files out of the way
		std::vector<std::string> moveBack;
		for (i = 0; i < group.size(); ++i)
		{
			if (IsFileRemovable(group.myAbsoluteFiles[i]))
			{
				std::string file = group.myAbsoluteFiles[i];
				std::string fileMoved = file + ".tortoise.removed";

				// Move the file out the way
				if (!MoveFile(file.c_str(), fileMoved.c_str()))
					DoMessageDialog("Couldn't move file:\n" + file + "\nto " + fileMoved);
				moveBack.push_back(file);

				args.add(group.myRelativeFiles[i]);
			}
		}

		// Let CVS do its stuff
		bool result = glue.Command(group.myDirectory, args);

		// Move the files back, recycling them if the command
		// was successful
		for (i = 0; i < moveBack.size(); ++i)
		{
			std::string file = moveBack[i];
			std::string fileMoved = file + ".tortoise.removed";

			// Move the file back
			if (!MoveFile(fileMoved.c_str(), file.c_str()))
				DoMessageDialog("Couldn't move file back:\n" + fileMoved + "\nto " + file);
			
			if (result)
			{
				// Move it to recycle bin
				SHFILEOPSTRUCT fileop;
				// this struct requires a *double* zero terminated string!
				std::string filename = file + '\0';
				fileop.hwnd     = 0; 
				fileop.wFunc    = FO_DELETE; 
				fileop.pFrom    = filename.c_str(); 
				fileop.pTo      = 0L; 
				fileop.fFlags   = FOF_ALLOWUNDO | FOF_SILENT | FOF_NOCONFIRMATION;
				if (SHFileOperation(&fileop) != 0)
					DoMessageDialog("Couldn't recycle:\n" + file);
			}
		}
	}
}

void TortoiseAct::PerformUpdateMenu()
{
	bool resetsticky = false;
	std::string getrevision;

	// If the ctrl key is held down, bring up dialog with
	// extra options for the update:
	if (IsControlDown())
	{
		bool query = DoUpdateDialog(resetsticky, getrevision);
		if (!query)	// cancel entire update operation
			return;
	}

	ASSERT(resetsticky == false || getrevision.empty());

	// When CVSAction is constructed, the dialog appears
	CVSAction glue;
	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("update");
		args.add("-d"); // create missing directories that exist in the repostiory

		if (resetsticky)
			args.add("-A");	// implies ("-P")
		else
		{
			args.add("-P"); // prune empty directories (i.e. if all the files in them are removed)
			if (!getrevision.empty())
			{
				args.add("-r");
				args.add(getrevision);
			}
		}

		for (unsigned int i = 0; i < group.size(); ++i)
			args.add(group.myRelativeFiles[i]);
		glue.Command(group.myDirectory, args);
	}

	// Clear the Explorer window (unless it was all directories being
	// updated, in which case no point)
	if (FileSelected() && glue.AtLeastOneSuccess())
		RefreshExplorerWindow();
}

void TortoiseAct::PerformWebLogMenu()
{
	// Only allow one file or directory
	if (mySingleAbsolute.empty())
		return;

	// Find the path on the server
	std::string ambientPart = "/" + CVSStatus::ServerPath(mySingleAbsolute) + "/";
	if (!IsDirectory(mySingleAbsolute.c_str()))
		ambientPart += ExtractLastPart(mySingleAbsolute);

	// Find server name
	std::string cvsroot = CVSStatus::CVSRootForPath(mySingleAbsolute);
	std::string server;
	if (CVSStatus::RootProtocol(cvsroot) != "local")
		server = CVSStatus::RootServer(cvsroot);
	else
		server = "localhost";

	// Scan and/or query user to find out what URL to use
	WebLog webLog(server, ambientPart);
	std::string url;
	bool result = webLog.PerformSearchAndQuery(url);

	// See if we failed to find anything (e.g. user cancelled)
	if (!result)
		return;
	
	// Detect plain log - which just does "cvs log" in the progress window
	if (url == "plainlog")
	{
		PerformLogMenu();
		return;
	}
	
	// Launch the URL
	DWORD Status = (DWORD)ShellExecute(NULL, NULL, url.c_str(), NULL, "", SW_SHOWNORMAL);
	if (Status <= 32)
	{
		std::string message = "Failed to launch\n" + url;
		MessageBox(NULL, message.c_str(), "TortoiseCVS", MB_OK);
	}
}

void TortoiseAct::PerformTagMenu( )
{
	std::string tagname;
	bool checkunmodified;
	if (DoTagDialog( tagname, checkunmodified ) )
	{
		CVSAction glue;
		for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
		{
			DirectoryGroup& group = myDirectoryGroups[j];

			MakeArgs args;
			args.add("tag");
			if( checkunmodified )
				args.add( "-c" );
			args.add( tagname );
			for (unsigned int i = 0; i < group.size(); ++i)
				args.add(group.myRelativeFiles[i]);
			glue.Command(group.myDirectory, args);
		}
	}
}


void TortoiseAct::PerformBranchMenu( )
{
	std::string branchname;
	bool checkunmodified;
	if (DoBranchDialog( branchname, checkunmodified ) )
	{
		CVSAction glue;
		for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
		{
			DirectoryGroup& group = myDirectoryGroups[j];

			MakeArgs args;
			args.add("tag");
			args.add( "-b" );
			if( checkunmodified )
				args.add( "-c" );
			args.add( branchname );
			for (unsigned int i = 0; i < group.size(); ++i)
				args.add(group.myRelativeFiles[i]);
			glue.Command(group.myDirectory, args);
		}
	}
}


void TortoiseAct::PerformMergeMenu()
{
	std::string fromtag;
	if (DoMergeDialog( fromtag ) )
	{

		CVSAction glue;
		for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
		{
			DirectoryGroup& group = myDirectoryGroups[j];

			MakeArgs args;
			args.add("update");
			args.add("-j");
			args.add( fromtag );
			for (unsigned int i = 0; i < group.size(); ++i)
				args.add(group.myRelativeFiles[i]);
			glue.Command(group.myDirectory, args);
		}

		if (FileSelected() && glue.AtLeastOneSuccess())
			RefreshExplorerWindow();
	}
}



void TortoiseAct::PerformHelp()
{
	std::string command = GetTortoiseFolder() + "Help.html";
	DWORD Status = (DWORD)ShellExecute(NULL, NULL, command.c_str(), NULL, "", SW_SHOWNORMAL);
	if (Status <= 32)
	{
		std::string message = "Failed to launch\n" + command;
		MessageBox(NULL, message.c_str(), "TortoiseCVS", MB_OK);
	}
}

void TortoiseAct::PerformPrefs()
{
	TortoisePreferences prefs;
	DoPreferencesDialog(prefs);
}

void TortoiseAct::PerformLogPropSheet()
{
	CVSAction glue(CVSAction::PIPE_STDOUT | CVSAction::HIDE_PROGRESS);
	PropSheetLogParser parser(glue.ConsoleOutput(), myRemoteHandle);

	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		MakeArgs args;
		args.add("log");
		for (unsigned int i = 0; i < group.size(); ++i)
		{
			CVSStatus::FileStatus status = CVSStatus::GetFileStatus(group.myAbsoluteFiles[i]);
			if( status == CVSStatus::STATUS_INCVS ||
				status == CVSStatus::STATUS_CHANGED ||
				status == CVSStatus::STATUS_CONFLICT )
			{
				args.add(group.myRelativeFiles[i]);
			}
		}
		glue.Command(group.myDirectory, args);
	}

	glue.SetCloseAnyway(true);

	// We shut the pipe, so the EOF reaches "parser" before its destructor.
	glue.CloseConsoleOutput();
}

void TortoiseAct::RefreshExplorerWindow()
{
	// This didn't work on Windows 98, only on NT
	// SendMessage(myRemoteHandle, WM_KEYDOWN, VK_F5, 0x003F0001); 
	// SendMessage(myRemoteHandle, WM_KEYUP, VK_F5, 0xC03F0001);

	// This is less preferable (F5 for refresh is less likely
	// to change than this command ID)
	PostMessage(myRemoteHandle, WM_COMMAND, 41504, 0);
}

// Returns true if they have selected at least one file
// Returns false if they have entirely selected directories
bool TortoiseAct::FileSelected()
{
	for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
	{
		DirectoryGroup& group = myDirectoryGroups[j];

		for (unsigned int i = 0; i < group.size(); ++i)
			if (!IsDirectory(group.myAbsoluteFiles[i].c_str()))
				return true;
	}

	return false;
}

void TortoiseAct::PerformMakePatchMenu()
{
	// Make patch file in temporary place
	std::string tempFilename = UniqueTemporaryFile();
	{
		CVSAction glue;
		glue.SetStreamFile(tempFilename);
		glue.SetCloseIfOK(true);
		for (unsigned int j = 0; j < myDirectoryGroups.size(); ++j)
		{
			DirectoryGroup& group = myDirectoryGroups[j];

			MakeArgs args;
			args.add("diff");
			args.add("-u");
			for (unsigned int i = 0; i < group.size(); ++i)
				args.add(group.myRelativeFiles[i]);
			if (!glue.Command(group.myDirectory, args))
				return;
		}
	}

	// See if any changes were made
	if (!FileExists(tempFilename.c_str()))
	{
		DoMessageDialog("There are no patches.\n\n"
				"All the files and folders that you\n"
				"selected are unchanged since you\n"
				"last performed a commit.");
		return;
	}

	// Query where to save the file
	std::string patchFilename = TortoiseRegistry::ReadString("Patch Save Name");
	patchFilename = DoSaveFileDialog("Save patch file", patchFilename, "Patch files (*.patch)|*.patch|Text files (*.txt)|*.txt|All files|*.*");
	if (patchFilename.empty())
		return;
	TortoiseRegistry::WriteString("Patch Save Name", patchFilename);
	DeleteFile(patchFilename.c_str());
	MoveFile(tempFilename.c_str(), patchFilename.c_str());

	// Launch it so the user can check it seems OK
	std::string command = "notepad \"" + patchFilename + "\"";
	LaunchCommand(command, false);
}

#define WinMain
