// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef TORTOISEACT_H
#define TORTOISEACT_H

#include <string>
#include <vector>

// We don't nest this struct inside TortoiseAct for 
// Visual C++ 5.0 compatibility.
struct DirectoryGroup
{
	std::string myDirectory;
	std::vector< std::string > myAbsoluteFiles;
	std::vector< std::string > myRelativeFiles;

	unsigned int size() { return myAbsoluteFiles.size(); }

#if _MSC_VER <= 1100
	// Need explicit operators for Visual C++ 5.0 compatibility:
	bool operator<(const DirectoryGroup& other) const
	{
		return (myDirectory < other.myDirectory) ||
				(myDirectory == other.myDirectory && myAbsoluteFiles < other.myAbsoluteFiles) || 
				(myDirectory == other.myDirectory && myAbsoluteFiles == other.myAbsoluteFiles && myRelativeFiles < other.myRelativeFiles);
	}
	bool operator==(const DirectoryGroup& other) const
	{
		return myDirectory == other.myDirectory && myAbsoluteFiles == other.myAbsoluteFiles && myRelativeFiles == other.myRelativeFiles;
	}
#endif
};

class TortoiseAct
{
public:
	TortoiseAct();

	// Returns true if args seem valid
	bool SussArgs( int ac, char* av[]);

	// Carries out the command
	void PerformAction();

private:
	void PerformAddMenu();
	void PerformAddMenuRecursive();
	void PerformCheckoutMenu();
	void PerformTortoiseCheckout();
	void PerformwxWindowsCheckout();
	void PerformMakeModuleMenu();
	void PerformCommitMenu();
	void PerformCommitDialogMenu();
	void PerformDiffMenu();
	void PerformMakePatchMenu();
	void PerformLogMenu();
	void PerformLogPropSheet();
	void PerformRemoveMenu();
	void PerformUpdateMenu();
	void PerformWebLogMenu();
	void PerformTagMenu();
	void PerformBranchMenu();
	void PerformMergeMenu();
	void PerformHelp();
	void PerformPrefs();

	bool PreprocessFileList(bool fancyRelativeNames = true);
	bool VerbNeedsFiles();
	void RefreshExplorerWindow();
	bool FileSelected();

	std::string FindRelativeName(std::string file, bool fancyRelativeNames);
	std::string FindDirectoryName(std::string file, bool fancyRelativeNames);
	
	// These are the full list of parameters passed in
	std::string myVerb;
	std::vector< std::string > myAllFiles;
	HWND myRemoteHandle;

	// These are grouped by directory
	// TODO: Separate out this grouping stuff into a separate "directory grouper"
	// class, which TortoiseAct contains.
	std::vector<DirectoryGroup> myDirectoryGroups;

	// For commands that require a single value
	std::string mySingleAbsolute;
	std::string mySingleDirectory;
	std::string mySingleRelative;
};


#endif
