// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - September 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef TORTOISEDIALOGS_H
#define TORTOISEDIALOGS_H
#include <string>
#include <vector>

bool DoBranchDialog( std::string& branchname, bool& checkunmodified );
bool DoTagDialog( std::string& tagname, bool& checkunmodified );
bool DoMergeDialog( std::string& fromtag );
bool DoUpdateDialog( bool& resetsticky, std::string& getrevision );
bool DoCheckoutDialog(std::string& protocol, std::string& userName,
		std::string& serverMachine, std::string& repositoryDirectory,
		std::string& module, std::string& tag, std::string& date );
bool DoMakeModuleDialog(std::string& protocol, std::string& userName,
		std::string& serverMachine, std::string& repositoryDirectory,
		const std::string& module);
bool DoCommitDialog(
		const std::vector<std::string>& modified, const std::vector<std::string>& added,
		const std::vector<std::string>& removed,
		std::string& comment, std::vector<std::string>& userSelection);

std::string DoOpenFileDialog(const std::string& titleMessage, const std::string& startBrowsingFrom, const std::string& patterns);
std::string DoSaveFileDialog(const std::string& titleMessage, const std::string& startBrowsingFrom, const std::string& patterns);

bool DoLogConfigDialog( std::string& urlStubPrefix, std::string& urlStubSuffix, const std::string& message, const std::string& regKey);

bool DoLoginDialog(std::string& password);

class ProgressDialogAbstract
{
public:
	virtual void LookBusy() = 0;
	virtual void NewText(char *txt, int len) = 0;
	virtual bool UserAborted() = 0;
	virtual void WaitForAbort() = 0;
	virtual bool Show(bool show) = 0;
};
ProgressDialogAbstract *CreateProgressDialog(bool show = true);
void DeleteProgressDialog(ProgressDialogAbstract *);

bool DoTextBinaryDialog(bool& text, const std::string& file);

bool DoMessageDialog(const std::string& message);

class TortoisePreferences;
void DoPreferencesDialog(TortoisePreferences& prefs);

#endif
