// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - October 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "TortoisePreferences.h"

TortoisePreferences::TortoisePreferences()
{
	// Fill in the details for each preference

    Preference closeIfNoError("Main", "Progress dialog", 
		"Automatically close the progress window when a command completes "
		"successfully.  You can hold Ctrl down while selecting a CVS "
		"command, or as one is finishing, to prevent this on a one off basis.", "Close If No Error");
    closeIfNoError.SetBoolean(false, "Automatically close if no error");
    myPrefs.push_back(closeIfNoError);

    Preference quietness("Main", "Progress messages",
		"How detailed CVS is about what it is doing.  Errors are always printed, "
		"even at the most silent levels.", "Quietness");
    std::vector<std::string> quietnessPossibilities;
    quietnessPossibilities.push_back("Loud");
    quietnessPossibilities.push_back("Quiet");
    quietnessPossibilities.push_back("Really Quiet");
    quietness.SetEnumeration(quietnessPossibilities, 1);
    myPrefs.push_back(quietness);

	Preference compression("Main", "Network compression", 
		"Reduce network load by compressing traffic to and from the "
		"CVS server. High levels may be slow to calculate.", "Compression Level");
    std::vector<std::string> compressionPossibilities;
    compressionPossibilities.push_back("0 - None");
    compressionPossibilities.push_back("3 - Low");
    compressionPossibilities.push_back("6 - Good");
    compressionPossibilities.push_back("9 - Best");
    compression.SetEnumeration(compressionPossibilities, 2);
    myPrefs.push_back(compression);

    Preference recalculateHome("Main", "Home directory", 
		"The home directory is used to cache CVS passwords for pserver "
		"connections. They are put in a file called .cvspass. "
		"SSH connections expect your .ssh folder with keys and known hosts "
		"to be in your home directory.\n"

		"By default, TortoiseCVS works out a home directory for you. "
		"It looks for directories in this order:\n"
		" - An environment variable called HOME\n"
		" - WinCVS specified home directory\n"
		" - My Documents\n"
		"You can turn off this automatic searching, and instead "
		"specify your own custom home directory below.",
		"Always Recalculate Home");
    recalculateHome.SetBoolean(true, "Always recalculate home directory");
    myPrefs.push_back(recalculateHome);

    Preference homeDirectory("Main", "Custom home directory", 
		"If you turn off automatic home directory calculation, you can enter "
		"an explict path here.", "HOME");
    homeDirectory.SetDirectory("C:");
	homeDirectory.SetGhostNegative(myPrefs.size() - 1);
    myPrefs.push_back(homeDirectory);

	Preference menuIcons("Quirky", "Menu icons",
		"Turn them off, or change their size",
		"ContextIcons");
	std::vector<std::string> menuIconPossibilities;
	menuIconPossibilities.push_back("None");
	menuIconPossibilities.push_back("Auto size");
	menuIconPossibilities.push_back("Small size");
	menuIconPossibilities.push_back("Large size");
	menuIcons.SetEnumeration(menuIconPossibilities, 1);
	myPrefs.push_back(menuIcons);

	Preference showChangedDirs("Quirky","Icon overlays",
		"Show directories which recursively contain changed files as changed.  This "
		"can be very slow.", "Show Changed Directories");
	showChangedDirs.SetBoolean(false, "Show changed directories");
	myPrefs.push_back(showChangedDirs);

	Preference clientLog("Quirky","Logging", 
		"Makes files cvsclient.log.in and cvsclient.log.out in the TortoiseCVS folder for debugging of "
		"the CVS protocol,", "Client Log");
	clientLog.SetBoolean(false, "Create client debug logs");
	myPrefs.push_back(clientLog);

    // Read initial values from registry
    for (unsigned int i = 0; i < myPrefs.size(); ++i)
    	myPrefs[i].ReadFromRegistry();
}

TortoisePreferences::~TortoisePreferences()
{
    // Write changed values to registry
    for (unsigned int i = 0; i < myPrefs.size(); ++i)
    	myPrefs[i].WriteToRegistry();
}

unsigned int TortoisePreferences::GetCount()
{
	return myPrefs.size();
}

Preference& TortoisePreferences::Lookup(int index)
{
	return myPrefs[index];
}

