// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
#ifndef PROP_SHEET_H
#define PROP_SHEET_H

#include <windows.h>

class CVSRevisionData;
class CVSRevisionDataSortCriteria;
class CVSRevisionsPropSheet;

// *********************** CVSRevisionData *************************
// One entry in the revisions list box
class CVSRevisionData
{
public:
	typedef enum
	{
		Revision,
		Symbolic,
		Branch
	} Type;

	CVSRevisionData(Type type, const std::string & revision, const std::string & tag, const std::string & branch, const struct tm *checkedIn, const std::string & author);
	virtual void GetDispInfo (NMLVDISPINFO* plvdi);

	Type type;
	std::string revision;
	std::string tag;
	std::string branch;
	std::string checkedIn;
	LONGLONG checkedInOrdinal;
	std::string author;
};

// *********************** CVSRevisionDataSortCriteria *************************
// How to sort the listview
class CVSRevisionDataSortCriteria
{
public:
	CVSRevisionDataSortCriteria() :
		key(-1), ascending(true), groupSameTypes(true) {}

	virtual void DoClick(int column);
	virtual void GetRegistryData();
	virtual void SaveRegistryData() const;

	int key;
	bool ascending;
	bool groupSameTypes;

	virtual int Compare(const CVSRevisionData &, const CVSRevisionData &) const;
	virtual int Compare(const std::string &, const std::string &) const;
	virtual int Compare(const LONGLONG &, const LONGLONG &) const;
	virtual int Compare(const CVSRevisionData::Type, const CVSRevisionData::Type) const;
};

// *********************** CVSRevisionsPropSheet *************************
// Displays and updates all the controls on the property sheet
class CVSRevisionsPropSheet
{
public:
	CVSRevisionsPropSheet(const std::string &filename);
	virtual ~CVSRevisionsPropSheet();

	virtual void SetHwnd(HWND hwnd);
	virtual BOOL PageProc(UINT uMessage, WPARAM wParam, LPARAM lParam);

	virtual const std::string &GetFilename() const { return filename; }
	virtual void AddRevision(CVSRevisionData const & revision);
	virtual int Compare(LPARAM, LPARAM) const;

protected:
	virtual void UpdateControls();
	virtual void InitWorkfileView();
	virtual void InitListView ();
	virtual void GetRegistryData();
	virtual void SaveRegistryData();

	HWND hwnd;
    HWND hwndListView;
	HIMAGELIST hwndImageList;
	const std::string filename;
	std::vector<CVSRevisionData> revisions;

	std::vector<int> nWidths;
	CVSRevisionDataSortCriteria sort;
};

#endif
