// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include "priv.h"

DEFINE_GUID(IID_IShellIconOverlayIdentifier,  0x0C6C4200L, 0xC589, 0x11D0, 0x99, 0x9A, 0x00, 0xC0, 0x4F, 0xD6, 0x55, 0xE1); 

DEFINE_SHLGUID(IID_IShellIconOverlayIdentifier,0x000214E5L,0,0);

// Initialize GUIDs (should be done only and at-least once per DLL/EXE)
#pragma data_seg(".text")
#define INITGUID
#include <initguid.h>
#include <shlguid.h>
#include "ShellExt.h"
#pragma data_seg()
#ifdef __MINGW32__
	#include <commctrl.h>
#endif

UINT      g_cRefThisDll = 0;    // Reference count of this DLL.
HINSTANCE g_hmodThisDll = NULL;	// Handle to this DLL itself.

extern "C" int APIENTRY
DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID /* lpReserved */)
{
    if (dwReason == DLL_PROCESS_ATTACH)
    {
        // Extension DLL one-time initialization
        g_hmodThisDll = hInstance;
	}
    else if (dwReason == DLL_PROCESS_DETACH)
    {
    }

    return 1;   // ok
}

STDAPI DllCanUnloadNow(void)
{
    return (g_cRefThisDll == 0 ? S_OK : S_FALSE);
}

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID *ppvOut)
{
    *ppvOut = NULL;

	TortoiseOLEClass whichClass = TORTOISE_OLE_INVALID;
    if (IsEqualIID(rclsid, CLSID_TortoiseCVS0))
		whichClass = TORTOISE_OLE_INCVS;
	else if (IsEqualIID(rclsid, CLSID_TortoiseCVS1))
		whichClass = TORTOISE_OLE_CHANGED;
	else if (IsEqualIID(rclsid, CLSID_TortoiseCVS2))
		whichClass = TORTOISE_OLE_NOTINCVS;
	else if (IsEqualIID(rclsid, CLSID_TortoiseCVS3))
		whichClass = TORTOISE_OLE_CONFLICT;

	if (whichClass != TORTOISE_OLE_INVALID)
	{
		CShellExtClassFactory *pcf = new CShellExtClassFactory(whichClass);

        return pcf->QueryInterface(riid, ppvOut);
    }

    return CLASS_E_CLASSNOTAVAILABLE;
}

CShellExtClassFactory::CShellExtClassFactory(TortoiseOLEClass classToMake)
{
	m_classToMake = classToMake;

    m_cRef = 0L;

    g_cRefThisDll++;	
}
																
CShellExtClassFactory::~CShellExtClassFactory()				
{
    g_cRefThisDll--;
}

STDMETHODIMP CShellExtClassFactory::QueryInterface(REFIID riid,
                                                   LPVOID FAR *ppv)
{
    *ppv = NULL;

    // Any interface on this object is the object pointer

    if (IsEqualIID(riid, IID_IUnknown) || IsEqualIID(riid, IID_IClassFactory))
    {
        *ppv = (LPCLASSFACTORY)this;

        AddRef();

        return NOERROR;
    }

    return E_NOINTERFACE;
}	

STDMETHODIMP_(ULONG) CShellExtClassFactory::AddRef()
{
    return ++m_cRef;
}

STDMETHODIMP_(ULONG) CShellExtClassFactory::Release()
{
    if (--m_cRef)
        return m_cRef;

    delete this;

    return 0L;
}

STDMETHODIMP CShellExtClassFactory::CreateInstance(LPUNKNOWN pUnkOuter,
                                                      REFIID riid,
                                                      LPVOID *ppvObj)
{
    *ppvObj = NULL;

    // Shell extensions typically don't support aggregation (inheritance)

    if (pUnkOuter)
    	return CLASS_E_NOAGGREGATION;

    // Create the main shell extension object.  The shell will then call
    // QueryInterface with IID_IShellExtInit--this is how shell extensions are
    // initialized.

    CShellExt* pShellExt = new CShellExt(m_classToMake);  //Create the CShellExt object

    if (NULL == pShellExt)
    	return E_OUTOFMEMORY;

    return pShellExt->QueryInterface(riid, ppvObj);
}


STDMETHODIMP CShellExtClassFactory::LockServer(BOOL /*fLock*/)
{
    return NOERROR;
}

// *********************** CShellExt *************************
CShellExt::CShellExt(TortoiseOLEClass tortoiseClass)
{
	m_TortoiseClass = tortoiseClass;

    m_cRef = 0L;
    m_pDataObj = NULL;

    g_cRefThisDll++;

	INITCOMMONCONTROLSEX used = {
		sizeof(INITCOMMONCONTROLSEX),
		ICC_LISTVIEW_CLASSES | ICC_WIN95_CLASSES
	};
	InitCommonControlsEx(&used);

	ContextMenuInitialise();
}

CShellExt::~CShellExt()
{
	ContextMenuFinalise();

    if (m_pDataObj)
        m_pDataObj->Release();

    g_cRefThisDll--;
}

STDMETHODIMP CShellExt::QueryInterface(REFIID riid, LPVOID FAR *ppv)
{
    *ppv = NULL;

    if (IsEqualIID(riid, IID_IShellExtInit) || IsEqualIID(riid, IID_IUnknown))
    {
    	*ppv = (LPSHELLEXTINIT)this;
    }
    else if (IsEqualIID(riid, IID_IContextMenu))
    {
        *ppv = (LPCONTEXTMENU)this;
    }
    else if (IsEqualIID(riid, IID_IContextMenu2))
    {
        *ppv = (LPCONTEXTMENU2)this;
    }
    else if (IsEqualIID(riid, IID_IContextMenu3))
    {
        *ppv = (LPCONTEXTMENU3)this;
    }
	else if (IsEqualIID(riid, IID_IShellIconOverlayIdentifier))
	{
		*ppv = (IShellIconOverlayIdentifier*)this;
	}
    else if (IsEqualIID(riid, IID_IShellPropSheetExt))
	{
        *ppv = (LPSHELLPROPSHEETEXT)this;
    }

    if (*ppv)
    {
        AddRef();

        return NOERROR;
    }

	return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG) CShellExt::AddRef()
{
    return ++m_cRef;
}

STDMETHODIMP_(ULONG) CShellExt::Release()
{
    if (--m_cRef)
        return m_cRef;

    delete this;

    return 0L;
}

STDMETHODIMP CShellExt::Initialize(LPCITEMIDLIST pIDFolder,
                                   LPDATAOBJECT pDataObj,
                                   HKEY /* hRegKey */)
{
    // Initialize can be called more than once
    if (m_pDataObj)
    	m_pDataObj->Release();

	// signal that there are no files to work on
	m_Files.clear();

	// duplicate the object pointer and registry handle
    if (pDataObj)
    {
    	m_pDataObj = pDataObj;
    	pDataObj->AddRef();

#if 0
		// extract filenames and store in array for later use
		m_Files.clear();
#endif

		FORMATETC fmte = {CF_HDROP, (DVTARGETDEVICE FAR *)NULL, DVASPECT_CONTENT, -1, TYMED_HGLOBAL };
		STGMEDIUM medium;
		HRESULT hres = m_pDataObj->GetData(&fmte, &medium);

		if (SUCCEEDED(hres))
		{
			// Number of files/folder selected
			UINT fileCount = 0;

			if (medium.hGlobal)
				fileCount = DragQueryFile((HDROP)medium.hGlobal, (UINT)-1, 0, 0);

			if (fileCount >= 1)
			{
				// Get the name of the file the user has clicked on
				if (fileCount)
				{
					char filename[_MAX_PATH + 1];
					for (unsigned int i = 0; i < fileCount; ++i)
					{
						DragQueryFile((HDROP)medium.hGlobal, i, filename, sizeof(filename));
						m_Files.push_back(filename);
					}
				}
			}
		}
	}
	else if (pIDFolder) // if a directory background
	{
		char dirname[_MAX_PATH + 1];

    	m_pDataObj = NULL; // just remember that it's empty

		// extract directory name and store in array for later use
		HRESULT hres = SHGetPathFromIDList(pIDFolder, dirname);
		if (SUCCEEDED(hres))
		{
			m_Files.push_back(dirname);
		}
	}

    return NOERROR;
}
