// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

//
// The class IDs of these Shell extension classs.
//
// class ids: 
//
// 5d1cb710-1c4b-11d4-bed5-005004b1f42f
// 5d1cb711-1c4b-11d4-bed5-005004b1f42f
// 5d1cb712-1c4b-11d4-bed5-005004b1f42f
// 5d1cb713-1c4b-11d4-bed5-005004b1f42f
//
//
// NOTE!!!  If you use this shell extension as a starting point, 
//          you MUST change the GUID below.  Simply run UUIDGEN.EXE
//          to generate a new GUID.
//
                                  
#ifndef SHELL_EXT_H
#define SHELL_EXT_H

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include <vector>
#include <string>
#include <map>

#include "../ContextMenus/MenuDescription.h"

extern	UINT		g_cRefThisDll; // Reference count of this DLL.
extern	HINSTANCE	g_hmodThisDll; // Instance handle for this DLL

DEFINE_GUID(CLSID_TortoiseCVS0, 0x5d1cb710L, 0x1c4b, 0x11d4, 0xbe, 0xd5, 0x00, 0x50, 0x04, 0xb1, 0xf4, 0x2f );
DEFINE_GUID(CLSID_TortoiseCVS1, 0x5d1cb711L, 0x1c4b, 0x11d4, 0xbe, 0xd5, 0x00, 0x50, 0x04, 0xb1, 0xf4, 0x2f );
DEFINE_GUID(CLSID_TortoiseCVS2, 0x5d1cb712L, 0x1c4b, 0x11d4, 0xbe, 0xd5, 0x00, 0x50, 0x04, 0xb1, 0xf4, 0x2f );
DEFINE_GUID(CLSID_TortoiseCVS3, 0x5d1cb713L, 0x1c4b, 0x11d4, 0xbe, 0xd5, 0x00, 0x50, 0x04, 0xb1, 0xf4, 0x2f );

enum TortoiseOLEClass
{
	TORTOISE_OLE_INVALID,
	TORTOISE_OLE_CHANGED,
	TORTOISE_OLE_CONFLICT,
	TORTOISE_OLE_NOTINCVS,
	TORTOISE_OLE_INCVS,
};

// this class factory object creates the main handlers -
// it's constructor says which OLE class it is to make
class CShellExtClassFactory : public IClassFactory
{
protected:
	TortoiseOLEClass m_classToMake;
	ULONG	m_cRef;

public:
	CShellExtClassFactory(TortoiseOLEClass classToMake);
	virtual ~CShellExtClassFactory();

	//IUnknown members
	STDMETHODIMP			QueryInterface(REFIID, LPVOID FAR *);
	STDMETHODIMP_(ULONG)	AddRef();
	STDMETHODIMP_(ULONG)	Release();

	//IClassFactory members
	STDMETHODIMP		CreateInstance(LPUNKNOWN, REFIID, LPVOID FAR *);
	STDMETHODIMP		LockServer(BOOL);

};

// The actual OLE Shell context menu handler
class CShellExt : public IContextMenu3,
                         IShellExtInit,
						 IShellIconOverlayIdentifier,
						 IShellPropSheetExt
{
protected:
	TortoiseOLEClass m_TortoiseClass;
	ULONG        m_cRef;
	LPDATAOBJECT m_pDataObj;
	std::vector<std::string> m_Files;
    char         m_szFileUserClickedOn[MAX_PATH];
	std::vector<MenuDescription> myMenus;

	std::map<std::string, int> myVerbMap;
	std::map<unsigned int, int> myIDMap;

public:
	CShellExt(TortoiseOLEClass tortoiseClass);
	virtual ~CShellExt();

	// IUnknown members
	STDMETHODIMP			QueryInterface(REFIID, LPVOID FAR *);
	STDMETHODIMP_(ULONG)	AddRef();
	STDMETHODIMP_(ULONG)	Release();

	// IContextMenu2 members
	STDMETHODIMP			QueryContextMenu(HMENU hMenu, UINT indexMenu, UINT idCmdFirst, UINT idCmdLast, UINT uFlags);
	STDMETHODIMP			InvokeCommand(LPCMINVOKECOMMANDINFO lpcmi);
	STDMETHODIMP			GetCommandString(UINT idCmd, UINT uFlags, UINT FAR *reserved, LPSTR pszName, UINT cchMax);
	STDMETHODIMP			HandleMenuMsg(UINT uMsg, WPARAM wParam, LPARAM lParam);
	void ContextMenuInitialise();
	void ContextMenuFinalise();

	// IContextMenu3 members
	STDMETHODIMP			HandleMenuMsg2(UINT uMsg, WPARAM wParam, LPARAM lParam, LRESULT *pResult);

	// IShellExtInit methods and related functions
	STDMETHODIMP		    Initialize(LPCITEMIDLIST pIDFolder, LPDATAOBJECT pDataObj, HKEY hKeyID);
	BOOL AreShellExtTargetsAvail() const { return(m_Files.size()>0); }

    // IExtractIcon methods
    STDMETHODIMP GetIconLocation(UINT uFlags, LPSTR szIconFile, UINT cchMax, int *piIndex, UINT *pwFlags);
    STDMETHODIMP Extract(LPCSTR pszFile, UINT nIconIndex, HICON *phiconLarge, HICON *phiconSmall, UINT nIconSize);

	// IShellIconOverlayIdentifier methods
	STDMETHODIMP GetOverlayInfo(LPWSTR pwszIconFile, int cchMax, int *pIndex, DWORD *pdwFlags);
	STDMETHODIMP GetPriority(int *pPriority);	
	STDMETHODIMP IsMemberOf(LPCWSTR pwszPath, DWORD dwAttrib);

	// IShellPropSheetExt methods
	STDMETHODIMP AddPages(LPFNADDPROPSHEETPAGE lpfnAddPage, LPARAM lParam);
    STDMETHODIMP ReplacePage (UINT, LPFNADDPROPSHEETPAGE, LPARAM);
};



#endif // SHELL_EXT_H
