// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - October 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "Preference.h"
#include "TortoiseRegistry.h"

Preference::Preference(const std::string& page,
	const std::string& description, 
	const std::string& tooltip, const std::string& registryKey)
: myDescription(description), myTooltip(tooltip),
	myRegistryKey(registryKey), myGhostPreference(-1),
	myGhostPositive(true), myPage(page)
{
}

void Preference::SetString(const std::string& defaultValue)
{
	myType = STRING;
	myStringData = defaultValue;
}

void Preference::SetDirectory(const std::string& defaultValue)
{
	myType = DIRPICK;
	myStringData = defaultValue;
}

void Preference::SetFile(const std::string& defaultValue)
{
	myType = FILEPICK;
	myStringData = defaultValue;
}

void Preference::SetEnumeration(const std::vector<std::string>& possibilities,
	int defaultValue)
{
	myType = ENUMERATION;
	myEnumerationPossibilities = possibilities;
	myEnumerationData = defaultValue;
}

void Preference::SetBoolean(bool defaultValue, const std::string& checkBoxText)
{
	myType = BOOLEAN;
	myBooleanData = defaultValue;
	myCheckBoxText = checkBoxText;
}

void Preference::ReadFromRegistry()
{
	switch(myType)
	{
	case STRING:
	case DIRPICK:
	case FILEPICK:
		myStringData = TortoiseRegistry::ReadString(myRegistryKey, myStringData);
		break;
	case ENUMERATION:
		myEnumerationData = TortoiseRegistry::ReadInteger(myRegistryKey, myEnumerationData);
		break;
	case BOOLEAN:
		myBooleanData = TortoiseRegistry::ReadBoolean(myRegistryKey, myBooleanData);
		break;
	}

}

void Preference::WriteToRegistry()
{
	switch(myType)
	{
	case STRING:
	case DIRPICK:
	case FILEPICK:
		TortoiseRegistry::WriteString(myRegistryKey, myStringData);
		break;
	case ENUMERATION:
		TortoiseRegistry::WriteInteger(myRegistryKey, myEnumerationData);
		break;
	case BOOLEAN:
		TortoiseRegistry::WriteBoolean(myRegistryKey, myBooleanData);
		break;
	}
}

void Preference::SetGhostPositive(int ghostPreference)
{
	myGhostPreference = ghostPreference;
	myGhostPositive = true;
}

void Preference::SetGhostNegative(int ghostPreference)
{
	myGhostPreference = ghostPreference;
	myGhostPositive = false;
}

