// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - October 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef PREFERENCE_H
#define PREFERENCE_H

#ifdef _MSC_VER
// turn off stupid debug-too-long warning
#pragma warning(disable:4786 4503)
#endif

#include <vector>
#include <string>

// Represents a single option for the user
class Preference
{
    public:
	// Construction
	Preference() {};
	Preference(const std::string& page, const std::string& description, const std::string& tooltip, const std::string& registryKey);
	void SetString(const std::string& defaultValue);
	void SetDirectory(const std::string& defaultValue);
	void SetFile(const std::string& defaultValue);
	void SetEnumeration(const std::vector<std::string>& possibilities, int defaultValue);
	void SetBoolean(bool defaultValue, const std::string& checkBoxText);

	// The sort of data that the preference is
	enum DATA_TYPE
	{
	    STRING,
		DIRPICK, // Type of STRING
		FILEPICK, // Type of STRING
	    ENUMERATION,
	    BOOLEAN
	};

	// Ghosting
	void SetGhostPositive(int ghostPreference);
	void SetGhostNegative(int ghostPreference);

	// Global properties
	DATA_TYPE myType;
	std::string myDescription;
	std::string myTooltip;
	std::string myRegistryKey;
	std::string myPage;
	int myGhostPreference;
	bool myGhostPositive;

	// Data specific to the type
	std::string myStringData;

	int myEnumerationData;
	std::vector<std::string> myEnumerationPossibilities;

	bool myBooleanData;
	std::string myCheckBoxText;

	// Storage
	void ReadFromRegistry();
	void WriteToRegistry();

#if _MSC_VER <= 1100
	// Need explicit operators for Visual C++ 5.0 compatibility:
	// Descriptions always differ, so we just compare them
	bool operator<(const Preference& other) const
	{
		return myDescription < other.myDescription;
	}
	bool operator==(const Preference& other) const
	{
		return myDescription == other.myDescription;
	}
#endif
};

#endif
