/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "PropSheetMsg.h"
#include <algorithm>

//**********************************************************
// sending
PropSheetMsgChannel::PropSheetMsgChannel()
{
	i = data.data();
}

bool PropSheetMsgChannel::Put(const void * _data, int size)
{
	const char * newData = (const char *) _data;
	data.append(newData, size);
	return true;
}

void PropSheetMsgChannel::Send(HWND receiver, HWND sender)
{
	COPYDATASTRUCT cds;
	cds.cbData = data.size();
	cds.lpData = (void *) data.data();
	cds.dwData = 0;
	SendMessage(receiver, WM_COPYDATA, (WPARAM) sender, (LPARAM) &cds);
}

PropSheetMsgChannel::PropSheetMsgChannel(const COPYDATASTRUCT *pcds)
{
	data.assign((const char *) pcds->lpData, pcds->cbData);
	i = data.data();
}

bool PropSheetMsgChannel::Get(void * _data, int size)
{
	memcpy(_data, i, size);
	i += size;
	return true;
}

//**********************************************************
#define MAGIC 0x1add1be2

PropSheetMsgHeader::PropSheetMsgHeader()
{
}

PropSheetMsgHeader::PropSheetMsgHeader(const std::string &_name) :
	name(_name)
{
}

bool PropSheetMsgHeader::Serialize(Serialized &s) const
{		
	return
		s.Put(MAGIC) &&
		s.Put(name);
}

bool PropSheetMsgHeader::Deserialize(Serialized &s)
{
	int magic;
	return
		s.Get(magic) &&
		magic == MAGIC &&
		s.Get(name);
}

//**********************************************************
bool PropSheetSymbolicMsg::IsMyHeader(const PropSheetMsgHeader &header)
{
	return header.name == "PropSheetSymbolicMsg";
}

bool PropSheetSymbolicMsg::Serialize(Serialized &s) const
{
	PropSheetMsgHeader header("PropSheetSymbolicMsg");
	return
		s.Put(header) &&
		s.Put(tag) &&
		s.Put(revisionNumber) &&
		s.Put(isBranch);
}

bool PropSheetSymbolicMsg::Deserialize(Serialized &s)
{
	return
		s.Get(tag) &&
		s.Get(revisionNumber) &&
		s.Get(isBranch);
}

//**********************************************************
bool PropSheetRevisionMsg::IsMyHeader(const PropSheetMsgHeader &header)
{
	return header.name == "PropSheetRevisionMsg";
}

bool PropSheetRevisionMsg::Serialize(Serialized &s) const
{
	PropSheetMsgHeader header("PropSheetRevisionMsg");
	return
		s.Put(header) &&
		s.Put(revisionNumber) &&
		s.Put(&timestamp, sizeof(timestamp)) &&
		s.Put(author);
}

bool PropSheetRevisionMsg::Deserialize(Serialized &s)
{
	return
		s.Get(revisionNumber) &&
		s.Get(&timestamp, sizeof(timestamp)) &&
		s.Get(author);
}
