/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef PropSheetMsg_H
#define PropSheetMsg_H

#include "Serialize.h"
#include <windows.h>
#include <time.h>

class PropSheetMsgChannel : public Serialized
{
public:
	// sending
	PropSheetMsgChannel();
	virtual bool Put(const int x) { return Serialized::Put(x); }
	virtual bool Put(const std::string &x) { return Serialized::Put(x); }
	virtual bool Put(const Serializable &x) { return Serialized::Put(x); }
	virtual bool Put(const void *, int size);
	virtual void Send(HWND receiver, HWND sender);

	// receiving
	PropSheetMsgChannel(const COPYDATASTRUCT *);
	virtual bool Get(int &x) { return Serialized::Get(x); }
	virtual bool Get(std::string &x) { return Serialized::Get(x); }
	virtual bool Get(Serializable &x) { return Serialized::Get(x); }
	virtual bool Get(void *, int size);

protected:
	std::string data;
	const char * i;
};

class PropSheetMsgHeader : public Serializable
{
public:
	std::string name;

	PropSheetMsgHeader();
	virtual bool Deserialize(Serialized &);

	PropSheetMsgHeader(const std::string &name);
	virtual bool Serialize(Serialized &) const;
};

class PropSheetSymbolicMsg : public Serializable
{
public:
	std::string tag;
	std::string revisionNumber;
	int isBranch;

	static bool IsMyHeader(const PropSheetMsgHeader &header);
	virtual bool Serialize(Serialized &) const;
	virtual bool Deserialize(Serialized &);
};

class PropSheetRevisionMsg : public Serializable
{
public:
	std::string revisionNumber;
	struct tm timestamp;
	std::string author;

	static bool IsMyHeader(const PropSheetMsgHeader &header);
	virtual bool Serialize(Serialized &) const;
	virtual bool Deserialize(Serialized &);
};

#endif
