// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2001 - Francis Irving
// <francis@flourish.org> - May 2001

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include "ShellUtils.h"
#include "TortoiseException.h"

LPITEMIDLIST GetNextItem(LPITEMIDLIST pidl)
{
	if (pidl == NULL)
		return NULL;

	// Get size of the item identifier we are on
	int cb = pidl->mkid.cb;

	// If zero, then end of list
	if (cb == 0)
		return NULL;

	// Move on
	pidl = (LPITEMIDLIST) (((LPBYTE)pidl) + cb);

	// Return NULL if null-terminating, or pidl otherwise
	return (pidl->mkid.cb == 0) ? NULL : (LPITEMIDLIST) pidl;

}

int GetItemCount(LPITEMIDLIST pidl)
{
	if (pidl == NULL)
		return 0;

	int count = 0;

	while ((pidl = GetNextItem(pidl)))
		count++;

	return count;
}

UINT GetSize(LPITEMIDLIST pidl)
{
	UINT total = 0;
	if (pidl)
	{
		total += sizeof(pidl->mkid.cb);
		while (pidl)
		{
			total += pidl->mkid.cb;
			pidl = GetNextItem(pidl);
		}
	}
	return total;
}

LPITEMIDLIST DuplicateItem(LPMALLOC pMalloc, LPITEMIDLIST pidl)
{
	int cb = pidl->mkid.cb;
	if (cb == 0)
		return NULL;

	LPITEMIDLIST pidlRet = (LPITEMIDLIST)pMalloc->Alloc(cb + sizeof(USHORT));
	if (pidlRet == NULL)
		return NULL;

	CopyMemory(pidlRet, pidl, cb);
	*((USHORT*) (((LPBYTE)pidlRet) + cb)) = 0;
	return pidlRet;
}

// From this Usenet article:
// http://groups.google.com/groups?hl=en&lr=&safe=off&ic=1&th=6075ee1dc2ec62e4,9&seekm=01bee26d%24aed10880%24e1c4f5d1%40ebenton#p
// http://groups.google.com/groups?q=HandleMenuMsg&hl=en&lr=&safe=off&rnum=2&ic=1&selm=38508C0C.122D923B%40cornell.edu

LPCONTEXTMENU2 pContextMenu2 = NULL;

bool DoExplorerMenu(HWND hwnd, LPCTSTR pszPath, POINT P)
{
    LPMALLOC pMalloc;
    LPSHELLFOLDER psfFolder, psfNextFolder;
    LPITEMIDLIST pidlMain, pidlItem, pidlNextItem;
    // Make sure the file name is fully qualified and in Unicode format.
    TCHAR tchPath[MAX_PATH];
    WCHAR wchPath[MAX_PATH];
    GetFullPathName(pszPath, sizeof(tchPath) / sizeof(TCHAR), tchPath, NULL);
    if (IsTextUnicode (tchPath, lstrlen (tchPath), NULL))
	{
        lstrcpy((char *)wchPath, tchPath);
	}
	else
	{
		MultiByteToWideChar(CP_ACP, 0, pszPath, -1, wchPath, sizeof(wchPath) / sizeof(WCHAR));
	}
	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
    bool bResult = false;
	if (!SUCCEEDED(SHGetMalloc(&pMalloc)))
		return bResult;
	if (!SUCCEEDED(SHGetDesktopFolder(&psfFolder)))
	{
		pMalloc->Release();
		return bResult;    
	}
	// Convert the path name into a pointer to an item ID list (pidl).
	unsigned long ulCount;
	if (SUCCEEDED(psfFolder->ParseDisplayName(hwnd, NULL, wchPath, &ulCount,
		&pidlMain, NULL)) &&
		(pidlMain != NULL))    
	{
        // nCount must be > 0
		unsigned int nCount = GetItemCount(pidlMain);
		if (nCount > 0)
		{
            // Initialize psfFolder with a pointer to the IShellFolder
			// interface of the folder that contains the item whose context
			// menu we're after, and initialize pidlItem with a pointer to
			// the item's item ID. If nCount > 1, this requires us to walk
			// the list of item IDs stored in pidlMain and bind to each
			// subfolder referenced in the list.
            pidlItem = pidlMain;
			while (--nCount)
			{
				// Create a 1-item item ID list for the next item in pidlMain.
				pidlNextItem = DuplicateItem(pMalloc, pidlItem);
				if (pidlNextItem == NULL)
				{
					pMalloc->Free(pidlMain);
					psfFolder->Release();
					pMalloc->Release();
					return bResult;
                }
				
				// Debugging
				//_STRRET debugStr;
				//_STRRET debugRelativeStr;
				//HRESULT debugResult = psfFolder->GetDisplayNameOf(pidlNextItem, SHGDN_NORMAL, &debugStr);
				//HRESULT debugResult2 = psfFolder->GetDisplayNameOf(pidlNextItem, SHGDN_INFOLDER, &debugRelativeStr);

				// Bind to the folder specified in the new item ID list.
				HRESULT ret = psfFolder->BindToObject(pidlNextItem, NULL,
					IID_IShellFolder,
					(LPVOID *)&psfNextFolder);
				if (!SUCCEEDED(ret) || !psfNextFolder)
				{
					pMalloc->Free(pidlNextItem);
                    pMalloc->Free(pidlMain);
                    psfFolder->Release();
                    pMalloc->Release();
                    return bResult;
                }
				// Release the IShellFolder pointer to the parent folder
				// and set psfFolder equal to the IShellFolder pointer for
				// the current folder.
				psfFolder->Release();
				psfFolder = psfNextFolder;
				// Release the storage for the 1-item item ID list we created
				// just a moment ago and initialize pidlItem so that it points
				// to the next item in pidlMain.
				pMalloc->Free(pidlNextItem);
				pidlItem = GetNextItem(pidlItem);
            }
			
			// Get a pointer to the item's IContextMenu interface and call
			// IContextMenu::QueryContextMenu to initialize a context menu.
			LPCONTEXTMENU pContextMenu;
			if (SUCCEEDED(psfFolder->GetUIObjectOf(hwnd, 1,
				(const ITEMIDLIST
				**)&pidlItem,
				IID_IContextMenu, NULL,
				(LPVOID *)&pContextMenu)))
			{
				// grab a pointer to the IContextMenu2 interface
				HRESULT hr = 
					pContextMenu->QueryInterface(IID_IContextMenu2,
					(LPVOID *)&pContextMenu2);
				if (SUCCEEDED(hr))
                {
					HMENU HPopupMenu = CreatePopupMenu();
					// fill the context menu (2) using the
					// IContextMenu::QueryContextMenu() function
					if (SUCCEEDED(pContextMenu2->QueryContextMenu(HPopupMenu,
						0, 1, 0x7FFF,
						CMF_EXPLORE)))
					{                        // Display the context menu.
						::ClientToScreen(hwnd, &P);
						unsigned int command;
						command = TrackPopupMenu(HPopupMenu,
							TPM_LEFTALIGN | 
							TPM_LEFTBUTTON |
							TPM_RIGHTBUTTON | 
							TPM_RETURNCMD,
							P.x, P.y, 0, 
							hwnd,  // use Form's handle
							NULL);
						// If a command was selected from the menu, execute it.
						if (command)
                        {
							CMINVOKECOMMANDINFO ici;
							ici.cbSize = sizeof(CMINVOKECOMMANDINFO);
							ici.fMask = 0;
							ici.hwnd = hwnd;
							ici.lpVerb = MAKEINTRESOURCE(command - 1);
							ici.lpParameters = NULL;
							ici.lpDirectory = NULL;
							ici.nShow = SW_SHOWNORMAL;
							ici.dwHotKey = 0;
							ici.hIcon = NULL;
							if (SUCCEEDED(pContextMenu2->InvokeCommand(&ici)))
								bResult = true;
						}
					}
					DestroyMenu(HPopupMenu);
					// release the pointer to IContextMenu2
					pContextMenu2->Release();
				}
				pContextMenu->Release();
			}
        }
		pMalloc->Free(pidlMain);    
	}
	// Clean up and return.
	psfFolder->Release();
	pMalloc->Release();
	pContextMenu2 = NULL;
	return bResult;
}

bool DoExplorerMenuWndProc(const MSG &msg)
{
	if (pContextMenu2 && (msg.message == WM_INITMENUPOPUP
		|| msg.message == WM_DRAWITEM || msg.message == WM_MEASUREITEM))
	{
		pContextMenu2->HandleMenuMsg(msg.message, msg.wParam, msg.lParam);
		return true;
	}

	return false;
}

bool LaunchCommand(const std::string& command, bool waitForEnd)
{
	PROCESS_INFORMATION processInfo;
	STARTUPINFO startupInfo;
	memset( &startupInfo, 0, sizeof(startupInfo) );
	startupInfo.cb = sizeof(startupInfo);
	int res = CreateProcess(NULL, (char*)command.c_str(), NULL, NULL, FALSE, 0, NULL, NULL, &startupInfo, &processInfo);
	if (res == 0)
	{
		return false;
	}

	if (waitForEnd)
		WaitForSingleObject(processInfo.hProcess, INFINITE);

	CloseHandle(processInfo.hProcess);
	CloseHandle(processInfo.hThread);
	return true;
}

