// $Id: SimpleMutex.h,v 1.5 2001/02/13 12:35:50 frabcus Exp $

// InstallLib - library of code for making Windows installation programs
// Copyright (c) 2001, CyberLife Technology Ltd.
// All rights reserved.
 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
// 
//  * Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//  * Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in
//    the documentation and/or other materials provided with the
//    distribution.
//  * Neither the name of CyberLife Technology Ltd. nor the names of its
//    contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//    
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL CYBERLIFE TECHNOLOGY 
// LTD OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// Mutex without frills

#ifndef SIMPLEMUTEX_H
#define SIMPLEMUTEX_H

#include <windows.h>

class SimpleMutex
{
public:

	SimpleMutex();
	~SimpleMutex();

	// Blocks the calling thread indefinitely until the mutex is available.
	// throws SimpleMutex::Err if anything goes wrong.
	void Lock();

	// Unlocks the mutex. Must be paired with Lock().
	void Release();

private:

	HANDLE myMutex;
};

// inlines:

inline SimpleMutex::SimpleMutex()
{
	myMutex = CreateMutex( NULL, false, NULL );
	if( myMutex == (HANDLE)ERROR_INVALID_HANDLE )
	{
		MessageBox(NULL, "Couldn't create Win32 mutex", "SimpleMutex", MB_OK);
	}
}

inline SimpleMutex::~SimpleMutex()
{
	if (CloseHandle( myMutex ) == 0)
	{
		MessageBox(NULL, "Couldn't close Win32 mutex", "SimpleMutex", MB_OK);
	}
}

inline void SimpleMutex::Lock()
{
	if( WaitForSingleObject( myMutex, INFINITE ) != WAIT_OBJECT_0 )
	{
		MessageBox(NULL, "Couldn't lock Win32 mutex", "SimpleMutex", MB_OK);
	}
}

inline void SimpleMutex::Release()
{
	if (ReleaseMutex(myMutex) == 0)
	{
		MessageBox(NULL, "Couldn't release Win32 mutex", "SimpleMutex", MB_OK);
	}
}

// Locks a mutex in its constructor, unlocks it in its destructor.
// Ensures you can't fail to unlock the mutex.

class SimpleMutexLock
{
public:
	SimpleMutexLock(SimpleMutex& simpleMutex)
		: mySimpleMutex(simpleMutex)
	{
		mySimpleMutex.Lock();
	}

	~SimpleMutexLock()
	{
		mySimpleMutex.Release();
	}

private:
	SimpleMutex& mySimpleMutex;
};

#endif // SIMPLEMUTEX_H
