// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include "TimeUtils.h"
#include <stdlib.h>

static const char * const months[] = {
	"Jan", "Feb", "Mar", "Apr", "May", "Jun",
	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
};
static const char * const daysOfWeek[] = {
	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
};

bool asctime_to_SYSTEMTIME(const char *asctime, SYSTEMTIME *lpdate)
{
	// "Wed Jan 02 02:03:55 1980"
	//	0000000000111111111122222
	//  0123456789012345678901234

	char str[26];
	int i;

    int year; 
    int month; 
    int dayOfWeek; 
    int day; 
    int hour; 
    int minute; 
    int second; 

	// check it's really an asctime string
	if (strlen(asctime) < 24 ||
		asctime[3] != ' ' ||
		asctime[7] != ' ' ||
		asctime[10] != ' ' ||
		asctime[13] != ':' ||
		asctime[16] != ':' ||
		asctime[19] != ' '
	)
		return false;

	// separate fields
	strcpy(str, asctime);
	str[3] = str[7] = str[10] = str[13] = str[16] = str[19] = str[24] = '\0';

	// search for day of week
	dayOfWeek = -1;
	for (i=0; i<7; i++) {
		if (strcmp(daysOfWeek[i], &str[0]) == 0) {
			dayOfWeek = i;
			break;
		}
	}

	// search for month
	month = -1;
	for (i=0; i<12; i++) {
		if (strcmp(months[i], &str[4]) == 0) {
			month = i;
			break;
		}
	}

	// scan numeric fields
	hour = atoi(&str[11]);
	day = atoi(&str[8]);
	minute = atoi(&str[14]);
	second = atoi(&str[17]);
	year = atoi(&str[20]);

	// check consistency
	if (
		dayOfWeek<0 || month<0 ||
		day<1 || day>31 ||
		hour<0 || hour >= 24 ||
		minute<0 || minute >= 60 ||
		second<0 || second >= 60 ||
		year<0 || year>=10000
	)
		return false;

	// save results
	lpdate->wYear = year;
	lpdate->wMonth = month+1;
	lpdate->wDayOfWeek = dayOfWeek;
	lpdate->wHour = hour;
	lpdate->wDay = day;
	lpdate->wMinute = minute;
	lpdate->wSecond = second;
	lpdate->wMilliseconds = 0;

	return true;
}

bool tm_to_SYSTEMTIME(const struct tm *ptm, SYSTEMTIME *pUtcTime)
{
	pUtcTime->wYear = ptm->tm_year + 1900;
	pUtcTime->wMonth = ptm->tm_mon;
	pUtcTime->wDayOfWeek = ptm->tm_wday;
	pUtcTime->wHour = ptm->tm_hour;
	pUtcTime->wDay = ptm->tm_mday;
	pUtcTime->wMinute = ptm->tm_min;
	pUtcTime->wSecond = ptm->tm_sec;
	pUtcTime->wMilliseconds = 0;
	return true;
}

bool SYSTEMTIME_to_local_SYSTEMTIME(const SYSTEMTIME *pUtcTime, SYSTEMTIME *lpdate)
{
	TIME_ZONE_INFORMATION tzi;
	if (GetTimeZoneInformation(&tzi) == TIME_ZONE_ID_INVALID)
		return false;

	if (!MySystemTimeToTzSpecificLocalTime(&tzi, pUtcTime, lpdate))
		return false;

	return true;
}

bool local_SYSTEMTIME_to_local_DateTimeFormatted(const SYSTEMTIME *pLocalTime, char *dateTimeFormatted, int fieldSize, bool longDateTime)
{
	char szTime[20];
	int dateLen;

	GetDateFormat(LOCALE_SYSTEM_DEFAULT, longDateTime?DATE_LONGDATE:DATE_SHORTDATE, pLocalTime, 0L, dateTimeFormatted, fieldSize);
	GetTimeFormat(LOCALE_SYSTEM_DEFAULT, longDateTime?0:TIME_NOSECONDS, pLocalTime, 0L, szTime, sizeof(szTime));
	dateLen = strlen(dateTimeFormatted);
	if (fieldSize-dateLen-2>0) {
		strncat(dateTimeFormatted, " ", fieldSize-dateLen-1);
		strncat(dateTimeFormatted, szTime, fieldSize-dateLen-2);
	}
	return true;
}

bool asctime_to_local_DateTimeFormatted(const char *asctime, char *dateTimeFormatted, int fieldSize, bool longDateTime)
{
	SYSTEMTIME utcTime;
	SYSTEMTIME localTime;
	return
		asctime_to_SYSTEMTIME(asctime, &utcTime) &&
		SYSTEMTIME_to_local_SYSTEMTIME(&utcTime, &localTime) &&
		local_SYSTEMTIME_to_local_DateTimeFormatted(&localTime, dateTimeFormatted, fieldSize, longDateTime);
}

bool tm_to_local_DateTimeFormatted(const struct tm *ptm, char *dateTimeFormatted, int fieldSize, bool longDateTime)
{
	SYSTEMTIME utcTime;
	SYSTEMTIME localTime;
	return
		tm_to_SYSTEMTIME(ptm, &utcTime) &&
		SYSTEMTIME_to_local_SYSTEMTIME(&utcTime, &localTime) &&
		local_SYSTEMTIME_to_local_DateTimeFormatted(&localTime, dateTimeFormatted, fieldSize, longDateTime);
}
