// TortoiseCVS - a Windows shell extension for easy version control

// Copyright (C) 2000 - Francis Irving
// <francis@flourish.org> - May 2000

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifdef _MSC_VER
#pragma warning(disable:4786 4503)
#endif

#include "TortoiseUtils.h"
#include "PathUtils.h"
#include "ShellUtils.h"
#include <windows.h>
#include <fstream>
#include <stdio.h>

#define va_ref_start(ap,v)  ( ap = (va_list)&v + _INTSIZEOF((&v)) )

std::string GetTortoiseFolder()
{
	// When in Explorer's process, we need the handle of our own dll.
	// Otherwise, just the default handle for the process will do.
	HMODULE module = ::GetModuleHandle("TortoiseShell.dll");
	if (!module)
		module = ::GetModuleHandle(NULL);

	// Convert into a directory
	char buf[_MAX_PATH + 1];
	::GetModuleFileName(module, buf, _MAX_PATH);
	std::string folder = GetDirectoryAbove(buf);
	EnsureTrailingDelimiter(folder);

	return folder;
}

void LaunchTortoiseAct(const std::vector<std::string>& files, const char * verbAndFormat, va_list args)
{
	// Write list of parameters out to a temporary file
	// (this is just an easy way to pass the data to TortoiseAct.exe
	// - sending the actual filenames on the command line wouldn't work
	// under Windows as there is quite a short length limit)
	std::string tempFile = UniqueTemporaryFile();
	std::ofstream out(const_cast<char*>(tempFile.c_str()), std::ios::out);
	for (unsigned int i = 0; i < files.size(); ++i)
		out << files[i] << std::endl;

	// Launch the external acting command with appropriate parameters
	std::string externalApp = GetTortoiseFolder() + "TortoiseAct.exe";
	std::string command = "\"" + externalApp + "\" ";
	{
		char verbAndOptions[100];
		vsprintf(verbAndOptions, verbAndFormat, args);
		command += verbAndOptions;
	}
	command += " -f " + tempFile;

	if (!LaunchCommand(command, false))
	{
		std::string message = "Failed to launch executable to perform action\n" + command;
		MessageBox(0, message.c_str(), "TortoiseCVS", MB_OK);
		return;
	}
}

void LaunchTortoiseAct(const std::vector<std::string>& files, const char * verbAndFormat, ...)
{
	va_list args;
	va_start(args, verbAndFormat);
	LaunchTortoiseAct(files, verbAndFormat, args);
	va_end(args);
}

void LaunchTortoiseAct(const std::string& verb, const std::vector<std::string>& files)
{
	LaunchTortoiseAct(files, verb.c_str());
}

void LaunchTortoiseAct(const std::string& verb, const std::string& file)
{
	std::vector<std::string> files;
	files.push_back(file);
	LaunchTortoiseAct(files, verb.c_str());
}

void LaunchTortoiseAct(const std::string& verb, const std::vector<std::string>& files, HWND hwnd)
{
	std::string verbAndFormat = verb + " -h 0x%lx";
	LaunchTortoiseAct(files, verbAndFormat.c_str(), hwnd);
}

void LaunchTortoiseAct(const std::string& verb, const std::string& file, HWND hwnd)
{
	std::vector<std::string> files;
	files.push_back(file);
	LaunchTortoiseAct(verb, files, hwnd);
}

static BOOL CALLBACK KillTortoiseActWindow(HWND hwnd, LPARAM lParam)
{
	SendMessage(hwnd,  WM_SYSCOMMAND,  (WPARAM) SC_CLOSE,  (LPARAM) 0);
	return true;
}
 