/*
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 1, or (at your option)
** any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef  CVSLOGBUILD_H
#define CVSLOGBUILD_H

#include "CvsLogParser.h"

class CvsLogBuild;

template <class PARSED>
class CvsLogBuildVectorParser : public Parser<PARSED>
{
public:
	CvsLogBuildVectorParser () : vect(0), last(0), parent(0), notifyStartPtr(0), notifyEndPtr(0) {}

	virtual void StartParse() { vect->push_back(PARSED()); last = &(*vect)[vect->size() - 1]; NotifyStart(**this); }
	virtual void EndParse() { NotifyEnd(**this); last= 0; }
	virtual bool IsParsing() { return last != 0; }
	virtual PARSED & operator*() { return *last; }
	virtual PARSED * operator->() { return last; }
	virtual void Reset() { if (vect != 0) vect->erase(vect->begin(), vect->end()); }

	typedef void (CvsLogBuild::*NotifyFuncPtr)(PARSED & _notify);

	virtual void SetVector(std::vector<PARSED> & _vect) { vect = &_vect; }
	virtual void SetNotify(CvsLogBuild & _parent, NotifyFuncPtr _notifyStartPtr = 0, NotifyFuncPtr _notifyEndPtr = 0)
	{
		parent = &_parent;
		notifyStartPtr = _notifyStartPtr;
		notifyEndPtr = _notifyEndPtr;
	}

protected:
	inline void NotifyStart(PARSED & _notify) { if (notifyStartPtr  != 0) (parent->*notifyStartPtr)(_notify); }
	inline void NotifyEnd(PARSED & _notify) { if (notifyEndPtr  != 0) (parent->*notifyEndPtr)(_notify); }

	std::vector<PARSED> *vect;
	PARSED *last;

	CvsLogBuild * parent;
	NotifyFuncPtr notifyStartPtr;
	NotifyFuncPtr notifyEndPtr;
};

class CvsLogBuild : public CvsLogParser
{
public:
	CvsLogBuild(std::vector<CRcsFile> & _rcsFiles);

	virtual void Reset();
	virtual Parser<CRcsFile> & RcsFile() { return rcsFile; }
	virtual Parser<CRevFile> & RevFile() { return revFile; }
	virtual Parser<CRevNumber> & Branch() { return branch; }
	virtual Parser<CRevNumber> & Symbolic() { return symbolic; }
	virtual Parser<CRevNumber> & Lock() { return lock; }
	virtual Parser<CLogStr> & Access() { return access; }

protected:
	CvsLogBuildVectorParser<CRcsFile> rcsFile;
	CvsLogBuildVectorParser<CRevFile> revFile;
	CvsLogBuildVectorParser<CRevNumber> branch;
	CvsLogBuildVectorParser<CRevNumber> symbolic;
	CvsLogBuildVectorParser<CRevNumber> lock;
	CvsLogBuildVectorParser<CLogStr> access;

	void NotifyStartRcsFile(CRcsFile &_rcsFile);
	void NotifyStartRevFile(CRevFile &_revFile);

	std::vector<CRcsFile> & rcsFiles;
};

#endif
